function [OUTPUT] = ASTRA_MODP(INPUT, seq)

% -------------------------------------------------------------------------
%
% MGA MULTI-OBJECTIVE (MO) OPTIMIZATION WITH DYNAMIC PROGRAMMING (DP)
%
% -------------------------------------------------------------------------

legs                                         = fromSeq2Legs(seq);  % --> find legs from the sequence
[vinflim, TOF_LIM, tstep, tofyMax, ~, ~, ~, ~,...
 costFunc1_MODP, costFunc2_MODP]             = check_INPUT(INPUT); % --> extract the limits on TOF and Vinf, and time step
INPUT.vInfLim                                = vinflim;            % --> update the INPUT
INPUT.tofyMax                                = tofyMax;            % --> update the INPUT
INPUT.costFunc1_MODP                         = costFunc1_MODP;     % --> update the INPUT
INPUT.costFunc2_MODP                         = costFunc2_MODP;     % --> update the INPUT

% --> run different optimizations for the chosen revolutions options
chosenRevs = INPUT.chosenRevs;
OUTPUT = struct('LEGSpf', [], 'VASpf', [], 'VINFpf', [], 'PF', [], 'LEGS', [], ...
    'VAS', [], 'VINFa', [], 'COSTS', [], 'TOFYS', [], 'minPATH', [], 'minCOST', [], ...
    'minTOFy', [], 'minDEFECT', [], 'minVINFd', [], 'minVINFa', [], 'chosenRevs', [], 'tocTOT', []);
for run = 1:size(chosenRevs,1)
    
    fprintf('Computing run : %d/%d \n', run, size(chosenRevs,1));
    
    % --> generate the Multi-Revolutions options
    runOpts = generateDiffRuns(chosenRevs(run,:), INPUT.res);
    
    % --> DYNAMIC PROGRAMMING : STEP 1
    T0                                     = [INPUT.depOpts(1):INPUT.depOpts(3):INPUT.depOpts(2)]'; % --> launch date vector
    tocV                                   = zeros(size(legs,1),1);
    [LEGSnext, VASnext, VINFnext, tocV(1)] = wrap_DynProgr_st1(T0, legs, runOpts, tstep, TOF_LIM, INPUT);
    
    % --> prune
    [LEGSnext, VASnext, VINFnext] = check_ISNAN_DP(LEGSnext, VASnext, VINFnext);
    [LEGSnext, VASnext, VINFnext] = prune_VINFn_DP(LEGSnext, VASnext, VINFnext, INPUT.vInfLim); % --> prune those exceeding the VINFn
    [LEGSnext, VASnext, VINFnext] = prune_TOFY_tot(LEGSnext, VASnext, VINFnext, INPUT.tofyMax); % --> prune those exceeding the total TOFy
    
    % --> DYNAMIC PROGRAMMING : STEP 2
    for indl = 2:size(legs,1)

        tic0 = tic;

        Nrev = runOpts(indl,:);
        if Nrev(3) ~= 0 % --> compute the resonant transfers
            [LEGSn, VASn, VINFn] = wrapConstructionResonance_DP(LEGSnext, VASnext, VINFnext, legs, Nrev(3:4), indl, deg2rad(1), INPUT.parallel);
        else % --> LP between two consecutive nodes
            [LEGSn, VASn, VINFn] = wrap_MODP_st2(LEGSnext, VASnext, legs, runOpts, indl, tstep, TOF_LIM, INPUT);
        end
        
        LEGSnext = LEGSn;
        VASnext  = VASn;
        VINFnext = VINFn;

        if isempty(LEGSn)        
            break
        end
        tocV(indl) = toc(tic0); toc(tic0)
        
    end
    tocTOT = sum(tocV);
    
    % --> save the results
    OUTPUT(run) = saveOUTPUT_MODP(LEGSnext, VASnext, VINFnext, INPUT, runOpts, chosenRevs(run,:), tocTOT);
    
end
hasNone           = cellfun(@isempty,{OUTPUT.LEGS});
OUTPUT(hasNone)   = [];

% --> plot the minimum DV path
try
    OUTPUT(1).TimeTOT = sum([OUTPUT.tocTOT]');
    if INPUT.plot(1) == 1
        [~, run] = min([OUTPUT.minCOST]');
        path     = OUTPUT(run).minPATH;
        plotPath(path);
    end
catch
end

% --> plot the Pareto front of all the solutions
try
    if INPUT.plot(2) == 1
        % --> compute the overall Pareto front
        LEGSnext      = cell2mat({OUTPUT.LEGSpf}');
        VASnext       = cell2mat({OUTPUT.VASpf}');
        VINFnext      = cell2mat({OUTPUT.VINFpf}');
        [~, ~, ~, PF] = INPUT.costFunc2_MODP(LEGSnext, VASnext, VINFnext);
        
        % --> save the overall Pareto front
        OUTPUT(1).LEGovPF  = LEGSnext(PF(:,3),:);
        OUTPUT(1).VASovPF  = VASnext(PF(:,3),:);
        OUTPUT(1).VINFovPF = VINFnext(PF(:,3),:);
        
        % --> plot the overall Pareto front
        figure; hold on; grid on;
        xlabel( 'Time of flight - years' ); ylabel(' \Deltav - km/s ');
        plot( PF(:,1), PF(:,2), 'o', 'MarkerEdgeColor', 'Black', 'MarkerFaceColor', 'Red' );
    end
catch
end

end