function [REACH, OUT] = checkDeltaMaxMin(vvIN, plIN, tIN)

% check the max and min energy orbits after the flyby
% - you can reach the min energy orbit only with an in-front passage of the
%   planet and only if alphaIN+deltaMAX>=pi
% - you can reach the max energy orbit only with a behind passage of the
%   planet and only if alphaIN-deltaMAX<=0
%
% what you are looking for is the closest possible alignment with the
% planet velocity post flyby (i.e. if minimum pericentre allows, the
% relative velocity vector would be aligned with the planet velocity vector
% or anti-velocity vector, i.e. minimising or maximising energy). This
% allows estimation of maximum apocentre/pericentre achievable post flyby,
% and hence reachable planet orbits.

mu = 132724487690;

[rrga, vvga] = EphSS_car(plIN, tIN);
vvInfIN      = vvIN - vvga;

% check maximum available delta (turning angle)
[muPL, radius] = planetConstants(plIN);
rpmin          = maxmin_flybyAltitude(plIN) + radius;
e_A            = 1 + (rpmin*(norm(vvInfIN)^2))/muPL;        % max. deflection hyperbola eccentricity
delta_A        = 2*asin(1/e_A);                             % max. deflection turning angle
delta_A        = wrapToPi(delta_A);                         % delta is positive in 180-degree range

alphain = acos(dot(vvInfIN, vvga)/(norm(vvInfIN)*norm(vvga)));
alphain = wrapToPi(alphain);                                % alphaIN is positive in 180-degree range

reach_max = double((alphain - delta_A) <= 0);
reach_min = double((alphain + delta_A) >= pi);

if reach_max == 0
    % you cannot reach the max energy orbit (alpha = 0) => closest possible
    % alignment
    alphaoumax     = alphain - delta_A;
    gam            = deg2rad(270);
    [~, vvscoumax] = swingby_vA(rrga, vvIN, plIN, tIN, gam, rpmin, muPL);
else
    % you can reach the max energy orbit
    alphaoumax = 0;
    vvinfoumax = norm(vvInfIN).*vvga./norm(vvga);
    vvscoumax  = vvinfoumax + vvga;
end

if reach_min == 0
    % you cannot reach the min energy orbit (alpha = 180) => closest possible
    % alignment
    alphaoumin     = alphain + delta_A;
    gam            = deg2rad(90);
    [~, vvscoumin] = swingby_vA(rrga, vvIN, plIN, tIN, gam, rpmin, muPL);
else
    % you can reach the min energy orbit
    alphaoumin = pi;
    vvinfoumin = norm(vvInfIN).*(-vvga./norm(vvga));
    vvscoumin  = vvinfoumin + vvga;
end

REACH(1,:) = [reach_max, reach_min];
REACH(2,:) = [alphain,   delta_A];
REACH(3,:) = rad2deg([alphain,   delta_A]);
REACH(4,:) = [REACH(3,1)-REACH(3,2), REACH(3,1)+REACH(3,2)];
REACH(5,:) = [alphaoumax,  alphaoumin];
REACH(6,:) = rad2deg([alphaoumax,  alphaoumin]);

kepmax = car2kep([rrga, vvscoumax], mu);
kepmin = car2kep([rrga, vvscoumin], mu);

rpmax = kepmax(1)*(1 - kepmax(2));
ramax = kepmax(1)*(1 + kepmax(2));
Emax  = -mu/(2*kepmax(1));

rpmin = kepmin(1)*(1 - kepmin(2));
ramin = kepmin(1)*(1 + kepmin(2));
Emin  = -mu/(2*kepmin(1));

OUT(1,:) = [rrga, vvscoumax, kepmax, rpmax, ramax, Emax];
OUT(2,:) = [rrga, vvscoumin, kepmin, rpmin, ramin, Emin];

end