function [MAT, INCL] = constructResonances_Vinf(pl, vInf, res, tpl, dincMax)

% This function computes resonant orbits provided an infinity velocity at a
% given planet.
%
% INPUT :
% - pl      : ID of the planet
% - vInf    : infinity velocity at the given planet
% - res     : vector with [N M], where N is the number of planet
%             revolutions and M is the number of spacecraft revolutions
% - tpl     : epoch of the flyby
% - dincMax : maximum inclination out of the ecliptic
%
% OUTPUT : 
% - MAT  : matrix containing initial and final states of the resonant orbit
%          and the time of the flybys
% - INCL : vector containing the inclination of the resonant orbit w.r.t.
%          the ecliptic plane
%
% -------------------------------------------------------------------------

mu = 132724487690;
% AU = 149597870.7;

[~, ~, smapl] = planetConstants(pl); % --> planet semi-major axis
Tpl = 2*pi*sqrt(smapl^3/mu);         % --> period of the planet
Tsc = res(1)/res(2)*Tpl;             % --> period of the SC
dt  = Tsc/86400;                     % --> transfer time (of the resonant orbit)
sma = (mu*(Tsc/(2*pi))^2)^(1/3);     % --> semi-major axis of the resonant orbit

% fig  = plotPLTS_tt(pl, tpl, tpl+dt);

% --> ephemerides of the planet at the two encounters
[rrga1, vvga1] = EphSS_car(pl, tpl);
[rrga2, vvga2] = EphSS_car(pl, tpl + dt);

phi = [deg2rad(0:5:360)]';

vscou1 = sqrt(mu*(2/norm(rrga1) - 1/sma)); % --> SC velocity after the first flyby

th = acos((vInf^2 + norm(vvga1)^2 - vscou1^2)/(2*vInf*norm(vvga1)));
if isreal(th) || abs(imag(th))<1e-4 % --> then a resonance exists
    
    th = real(th);
    th = wrapTo2Pi(th);
    
    alpha = pi - th;

    vInfIN2 = zeros(length(phi), 1);
    vvIN2   = zeros(length(phi), 3);
    kepIN2  = zeros(length(phi), 6);
    rrscou1 = zeros(length(phi), 3);
    vvscou1 = zeros(length(phi), 3);
    
    for indphi = 1:length(phi)

        PHI      = phi(indphi);
        vvInfOU1 = vInf.*[cos(alpha), sin(alpha)*cos(PHI), -sin(alpha)*sin(PHI)];

        vvers = vvga1./norm(vvga1);
        nvers = cross(rrga1, vvga1)./norm(cross(rrga1, vvga1));
        cvers = cross(vvers, nvers);
        M     = [vvers' nvers' cvers'];

        vvInfOU1 = M*vvInfOU1';
        vvInfOU1 = vvInfOU1';

        vvInfIN2           = vvInfOU1 + vvga1 - vvga2;
        vvIN2(indphi,:)    = vvInfIN2 + vvga2;

        rrscou1(indphi,:)  = rrga1;
        vvscou1(indphi,:)  = vvInfOU1 + vvga1;
        kepscou1           = car2kep([rrscou1(indphi,:), vvscou1(indphi,:)], mu);
        [~, vvf]           = FGKepler_dt(kepscou1, dt*86400, mu);
        vvIN2(indphi,:)    = vvf;

        vInfIN2(indphi,:) = norm(vvInfIN2);
        kepIN2(indphi,:)  = car2kep([rrga2, vvIN2(indphi,:)], mu);
        
%         % --> plot the trajectories
%         tt         = [0:1*86400:dt*86400]';
%         [~,yy]     = propagateKepler(rrscou1(indphi,:), vvscou1(indphi,:), tt, mu); % forward propagation
%         hold on;
%         plot3(yy(:,1)./AU, yy(:,2)./AU, yy(:,3)./AU, 'b', 'linewidth', 2, 'handlevisibility', 'off');
%         if rad2deg(kepIN2(indphi,3)) > dincMax
%             plot3(yy(:,1)./AU, yy(:,2)./AU, yy(:,3)./AU, 'r', 'linewidth', 2, 'handlevisibility', 'off');
%         end
        
        
    end
    
    % --> save the results
    MAT = [rrscou1 vvscou1 tpl.*ones(size(rrscou1,1),1) ...
        rrga2.*ones(size(rrscou1,1),3) vvIN2 (tpl+dt).*ones(size(rrscou1,1),1)];
    INCL = rad2deg(kepIN2(:,3));
    
    % --> eliminate those solutions with too high inclination w.r.t. the eclipltic plane
    indxs = find(INCL > dincMax);
    MAT(indxs,:)  = [];
    INCL(indxs,:) = [];
    
else
    MAT  = [];
    INCL = [];
end

end