function [VI,VF] = lambert_ZeroRevs(RI, RF, TOF, MU)

    nitermax = 2000; % Maximum number of iterations for loops
    TOL      = 1e-14;

    TWOPI = 2*pi;
    PI    = TWOPI/2;

    % Reset
    VI = [0, 0, 0];
    VF = [0, 0, 0];

    % ----------------------------------
    % Compute the vector magnitudes and various cross and dot products

    RIM2   = dot(RI,RI);
    RIM    = sqrt(RIM2);
    RFM2   = dot(RF,RF);
    RFM    = sqrt(RFM2);
    CTH    = dot(RI,RF)/(RIM*RFM);
    CR     = cross(RI,RF);
    STH    = norm(CR)/(RIM*RFM);
    if CR(3) < 0 % always direct transfer
        STH = -STH;
    end

    THETA  = (atan2(STH,CTH));
    THETA = THETA - (TWOPI * (fix(THETA/TWOPI) + min([0,sign(THETA)])));

    if THETA == TWOPI || THETA == 0
        VI=[0,0,0]; VF=[0,0,0]; 
        return
    end

    B1     = sign(STH); if STH == 0; B1 = 1; end

    % ----------------------------------
    % Compute the chord and the semi-perimeter

    C = sqrt(RIM2 + RFM2 - 2*RIM*RFM*CTH);
    S = (RIM + RFM + C)/2;
    LAMBDA = B1*sqrt((S-C)/S);

    if 4*TOF*LAMBDA == 0 || abs((S-C)/S) < TOL
        VI=[0,0,0]; VF=[0,0,0]; 
        return
    end

    % ----------------------------------
    % Compute L carefully for transfer angles less than 5 degrees

    if THETA*180/PI <= 5
       W   = atan((RFM/RIM)^.25) - PI/4;
       R1  = (sin(THETA/4))^2;
       S1  = (tan(2*W))^2;
       L   = (R1+S1)/(R1+S1+cos(THETA/2));
    else
       L   = ((1-LAMBDA)/(1+LAMBDA))^2;
    end

    M      = 8*MU*TOF^2/(S^3*(1+LAMBDA)^6);
    TPAR   = (sqrt(2/MU)/3)*(S^1.5-B1*(S-C)^1.5);
    L1     = (1 - L)/2;

    CHECKFEAS = 0;

    % ----------------------------------
    % Initialize values of y, n, and x

    Y  = 1;
    N  = 0;
    N1 = 0;

    if (TOF-TPAR) <= 1e-3
        X0  = 0;
    else
        X0  = L;
    end

    X= -1.e8;

    % ----------------------------------
    % Begin iteration

    while (abs(X0-X) >= abs(X)*TOL+TOL) && (N <= nitermax)
        N   = N+1;
        X   = X0;
        ETA = X/(sqrt(1+X) + 1)^2;
        CHECKFEAS=1;

        % ----------------------------------
        % Compute x by means of an algorithm devised by
        % Gauticci for evaluating continued fractions by the
        % 'Top Down' method

        DELTA = 1;
        U     = 1;
        SIGMA = 1;
        M1    = 0;

        while abs(U) > TOL && M1 <= nitermax
            M1    = M1+1;
            GAMMA = (M1 + 3)^2/(4*(M1+3)^2 - 1);
            DELTA = 1/(1 + GAMMA*ETA*DELTA);
            U     = U*(DELTA - 1);
            SIGMA = SIGMA + U;
        end

        C1 = 8*(sqrt(1+X)+1)/(3+1/(5 + ETA + (9*ETA/7)*SIGMA));

        % ----------------------------------
        % Compute H1 and H2

        if N == 1
            H1 = (L+X)^2*(C1 + 1 + 3*X)/((1 + 2*X + L)*(3*C1 + X*C1 +4*X));
            H2 = M*(C1+X-L)/((1 + 2*X + L)*(3*C1 + X*C1 +4*X));
        else
            QR = sqrt(L1^2 + M/Y^2);
            H1 = (((QR - L1)^2)*(C1 + 1 + 3*X))/((2*QR)*(3*C1 + X*C1+4*X));
            H2 = M*(C1+X-L)/((2*QR)*(3*C1 + X*C1+4*X));
        end

        B = 27*H2/(4*(1+H1)^3);
        U = -B/(2*(sqrt(B+1)+1));

        % ----------------------------------
        % Compute the continued fraction expansion K(u)
        % by means of the 'Top Down' method

        DELTA = 1;
        U0    = 1;
        SIGMA = 1;
        N1    = 0;

        while N1 < nitermax && abs(U0) >= TOL
            if N1 == 0
                GAMMA = 4/27;
                DELTA = 1/(1-GAMMA*U*DELTA);
                U0 = U0*(DELTA - 1);
                SIGMA = SIGMA + U0;
            else
                for I8 = 1:2
                    if I8 == 1
                        GAMMA = 2*(3*N1+1)*(6*N1-1)/(9*(4*N1 - 1)*(4*N1+1));
                    else
                        GAMMA = 2*(3*N1+2)*(6*N1+1)/(9*(4*N1 + 1)*(4*N1+3));
                    end
                    DELTA = 1/(1-GAMMA*U*DELTA);
                    U0 = U0*(DELTA-1);
                    SIGMA = SIGMA + U0;
                end
            end

            N1 = N1 + 1;
        end

        KU = (SIGMA/3)^2;
        Y = ((1+H1)/3)*(2+sqrt(B+1)/(1-2*U*KU));    % Y = Ycami

        X0 = sqrt(((1-L)/2)^2+M/Y^2)-(1+L)/2;
    end

    % ----------------------------------
    % Compute the velocity vectors

    if CHECKFEAS == 0
        VI=[0,0,0]; VF=[0,0,0]; 
        return
    end

    if N1 >= nitermax || N >= nitermax
        VI=[0,0,0]; VF=[0,0,0];
        return
    end

    R11 = (1 + LAMBDA)^2/(4*TOF*LAMBDA);
    S11 = Y*(1 + X0);
    T11 = (M*S*(1+LAMBDA)^2)/S11;

    VI(1) = -R11*(S11*(RI(1)-RF(1))-T11*RI(1)/RIM);
    VI(2) = -R11*(S11*(RI(2)-RF(2))-T11*RI(2)/RIM);
    VI(3) = -R11*(S11*(RI(3)-RF(3))-T11*RI(3)/RIM);

    VF(1) = -R11*(S11*(RI(1)-RF(1))+T11*RF(1)/RFM);
    VF(2) = -R11*(S11*(RI(2)-RF(2))+T11*RF(2)/RFM);
    VF(3) = -R11*(S11*(RI(3)-RF(3))+T11*RF(3)/RFM);

    

end