function [MOIDa, MOIDd] = PhD_MOIDapproxBonnano3(orbElem1, orbElem2)
% function [MOIDa  MOIDd] = PhD_MOIDapproxBonnano3(orbElem)
%**************************************************************************
% PhD_MOIDapproxBonnano3 V 1.0 *  FUNCTION :  PhD_MOIDapproxBonnano3
%*******************************  AUTHOR   :  Daniel Garcia Yarnoz (dgy)
% STRATH/DME/ASCL              *  PROJECT  :  MechanicsAsteroids
% PHD_FUNCTIONS                *  MATLAB   :  V 7.13.0.564 (R2011b)
%**************************************************************************
%
%    XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
%    X                                                                X
%    X                  MOID APPROXIMATION BY BONNANO (3)             X
%    X                  ---- ------           -------  -              X
%    XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
%
%   It calculates an approximation to the minimum orbit intersection 
%   distance between an orbit and the Earth. Earth has inclination zero
%   (orbital elemnts given with respect to plane of Earth)
%   Based on Bonanno, C. (2000). "An analytical approximation for the MOID 
%   and its consequences." Astronomy and Astrophysics 360: 411-416.
%   Not finished
%
%**************************************************************************
% HISTORY : V 1.0     20/10/2011 -dgy- creation
%           V 1.1     21/20/2011 -dgy- Earth orb elem optional
%**************************************************************************
% INPUTS  : orbElem   (1x6) [AU,-,rad]   Orbital elements of asteroid
%           earthElem (1x6) [AU,-,rad]   Orbital elements of earth (opt.)
%
% OUTPUTS : MOIDa   (1x1)   [AU]   MOID in ascending node (valid if small)
%           MOIDd   (1x1)   [AU]   MOID in descending node (valid if small)
%           
%**************************************************************************

%% Initialisation: orbital elements
sma    = orbElem1(1);    % in AU
ecc    = orbElem1(2);
inc    = orbElem1(3);    % all angles in rad
w_p    = orbElem1(5);
p      = sma*(1-ecc^2);

if nargin <2
    orbElem2    = [1 0 0 0 0 0];  % If orbital elements not provided, 
                                   % Earth assumed in 1 AU circular orbit
end
smaE   = orbElem2(1);  % in AU
eccE   = orbElem2(2);  % all angles in rad
w_pE   = orbElem2(5);
pE     = smaE*(1-eccE^2);

%% Calculation

% Earth and SC/asteroid distances at the common node
rEa    = pE/(1+eccE*cos(w_pE));
rEd    = pE/(1-eccE*cos(w_pE));

rSCa   = p/(1+ecc*cos(w_p));
rSCd   = p/(1-ecc*cos(w_p));

% Nodal distance:
x0_asc = rSCa - rEa;
x0_des = rSCd - rEd;

% Velocity of SC/asteroid at node
vSCa   = [sqrt(2/rSCa-1/sma-p/rSCa^2) sqrt(p)/rSCa*[cos(inc) sin(inc)]];
vSCd   = [sqrt(2/rSCd-1/sma-p/rSCd^2) sqrt(p)/rSCd*[cos(inc) sin(inc)]];

% Velocity of the Earth at node
alphaa = acos(sqrt(smaE*pE/rEa/(2*smaE-rEa)));
vEa    = sqrt(2/rEa-1/smaE)*[ sin(alphaa) cos(alphaa) 0];
alphad = acos(sqrt(smaE*pE/rEd/(2*smaE-rEd)));
vEd    = sqrt(2/rEd-1/smaE)*[ sin(alphad) cos(alphad) 0];

% Cross product
Ta     = cross(vSCa,vEa);
Td     = cross(vSCd,vEd);

% --> Distance (MOID)
MOIDa  = sqrt(x0_asc^2*Ta(1)^2/norm(Ta)^2);
MOIDd  = sqrt(x0_des^2*Td(1)^2/norm(Td)^2);

end
