function [x_sunline,sun_ref] = eci2sunline(x_eci,y_ref,lambda)

% eci2sunline.m - Vector reference frame transformation.
%   ECI reference frame to sun-line reference frame.
%
% PROTOTYPE:
%   x_sunline = eci2sunline(x_eci,y_ref,lambda)
%
% DESCRIPTION:
%   Transformation from ECI to sun-line reference frame.
%   ECI reference frame: {c1,c2,c3}
%       (c1,c2) plane: equatorial plane.
%       c3-axis: direction of the North Pole from the centre of the Earth.
%   Sun-line reference frame: {rs,p,rs_p} - as in McInnes, p.115
%       rs-axis: direction of the Sun. The orientation is from the Sun to
%                the spacecraft.
%       p-axis: perpendicular to the sun-line direction, in the plane
%               formed by rs-axis and y_ref-vector.
%       rs_p-axis: completes the reference frame, it is the vector
%                  perpendicular both to rs-axis and y_ref-vector.
%
% INPUT:
%	x_eci[3]    Vector to be transformed, expressed in {c1,c2,c3}.
%   y_ref[3]    Reference vector to define the sun-line reference
%                frame, expressed in {c1,c2,c3}.
% 	lambda      Longitude of the Sun [rad].
%
% OUTPUT:
%	x_sunline[3,1]  Vector transformed into {rs,p,rs_p}.
%
% EXAMPLE:
%   Given a sailcraft in orbit:
%       - we have the velocity vector in ECI;
%       - we want to optimize the force due to SRP and atmospheric drag in
%       sun-line reference frame.
%   In this case:
%       x_eci = Velocity vector in ECI;
%       y_ref = Velocity vector in ECI;
%       x_sunline = Velocity vector, transformed in {rs,p,rs_p}.
%
% CALLED FUNCTIONS:
%   crossFast
%
% AUTHOR:
%   Alessandro Peloni, 11/05/2013, MATLAB, eci2sunline.m
%
% CHANGELOG:
%   25/06/2013, Alessandro Peloni: change the sign of the sun-line unit
%       vector.
%
% -------------------------------------------------------------------------

x_eci = x_eci(:);

epsilon = 23.5*pi/180; % Obliquity of the equator on the ecliptic plane [rad]

% Unit vector from the Sun to the spacecraft
rs = [cos(lambda) sin(lambda)*cos(epsilon) sin(lambda)*sin(epsilon)]';

rs_p = crossFast(rs,y_ref);
rs_p = rs_p/norm(rs_p);

p = crossFast(rs_p,rs);
p = p/norm(p);

R = [rs p rs_p];

x_sunline = R'*x_eci;
sun_ref = [rs p rs_p];

return