function kep = ev2kep(ev,lambda,Omega_0)

% ev2kep.m -  Equinoctial Variables to Keplerian elements.
%
% PROTOTYPE:
%   kep = ev2kep(ev,lambda,Omega_0)
%
% DESCRIPTION:
%   Transformation from Equinoctial Variables to Keplerian elements.
%   Vector of Equinoctial Variables:
%       [a P1 P2 Q1 Q2 l], where l is the mean longitude.
%       a in [m], l in [rad].
%
%   Vector of Keplerian elements:
%       [a e i Om om f], where f is the true anomaly.
%       a in [m], angles in [rad].
%   The threshold for inclination is set to 0.1deg, so if incl < 0.1deg -->
%    incl = 0.
% 
% INPUT:
%	ev[6]       Vector of Equinoctial Variables [a P1 P2 Q1 Q2 l],
%                where l is the mean longitude. a in [m], l in [rad].
%   lambda      Longitude of the Sun respect to Cartesian coordinates [rad].
%   Omega_0     Initial value of the Right Ascension of the Ascending Node
%                [rad].
% 
% OUTPUT:
%	kep[6,1]    Vector of Keplerian elements [a e i Om om f],
%                where f is the true anomaly. a in [m], angles in [rad].
%
%
% CALLED FUNCTIONS:
%   astroCostants, kepEq_f
%
% AUTHOR:
%   Alessandro Peloni, 11/05/2013, MATLAB, ev2kep.m
%
% CHANGELOG:
%   01/07/2013, Alessandro Peloni: change the condition on eccentricity and
%       inclination for the calculus of omega and Omega.
%   03/07/2013, Alessandro Peloni: add lambda input, for the calculus of
%       the value of the argument of perigee when e << 1.
%   05/07/2013, Alessandro Peloni: add error if the eccentricity is major
%       than 1, because the true anomaly is defined only for elliptic
%       orbits.
%   08/07/2013, Alessandro Peloni: changes in calculus of Omega, omega, f.
%   20/09/2013, Alessandro Peloni: set the threshold for both eccentricity
%       and inclination to zero. To avoid the problem with the perfectly
%       circular or equatorial orbit, set the initial value of eccentricity
%       or inclination respectively not to zero, but to a value close to
%       zero.
%   09/10/2013, Alessandro Peloni: Add the initial value of Omega to
%       generalize the case of equatorial orbits.
%
% -------------------------------------------------------------------------

mu = astroConstants(13) * 1e9; % [m^3/s^2]

th1 = 0.1 *pi/180;  % threshold for inclination [rad]
th2 = 0;  % threshold for eccentricity

kep(1,1) = ev(1); % semi-major axis

kep(2,1) = sqrt(ev(2)^2+ev(3)^2); % eccentricity

kep(3,1) = 2*atan(sqrt(ev(4)^2+ev(5)^2)); % inclination
if kep(3,1) < 0 % inclination is defined into [0,pi]
    kep(3,1) = kep(3,1)+pi;
end

if kep(3,1) > th1
    kep(4,1) = atan2(ev(4)/tan(kep(3,1)/2), ev(5)/tan(kep(3,1)/2)) + lambda; % RAAN
    kep(4,1) = mod(kep(4,1),2*pi); % RAAN in [0,2pi]
else
    kep(4,1) = mod(lambda + Omega_0,2*pi);
end

if kep(2,1) > th2
    om_t = mod(atan2(ev(2)/kep(2,1), ev(3)/kep(2,1)),2*pi); % omega_tilde = om + Om
    kep(5,1) = om_t - kep(4,1); % argument of perigee
    kep(5,1) = mod(kep(5,1),2*pi); % omega in [0,2pi]
else
    om_t = kep(4,1);
%     kep(5,1) = wrapTo2Pi(lambda + 3/2*pi - kep(4,1));
    kep(5,1) = 0;
end

t = (ev(6)-om_t)*sqrt(ev(1)^3/mu);

% if kep(2,1) >= 1
%     save errorFlag
%     error('Eccentricity is greater than 1, cannot continue with the calculus of the true anomaly')
% end

[f,errorflag] = kepEq_f(t,ev(1),kep(2,1),mu,0,0); % true anomaly
if errorflag == 2
    warning(['errorflag for the calculus of the true anomaly: ', num2str(errorflag), ...
        '. The orbit is not elliptical.'])
    toc
    f = 0;
end
kep(6,1) = f;

return