function mee = kep2mee(kep)

% kep2mee.m -  Keplerian Elements to Modified Equinoctial Elements.
%
% PROTOTYPE:
%   mee = kep2mee(kep)
%
% DESCRIPTION:
%   Transformation from Keplerian elements to Modified Equinoctial Elements.
%
%   Vector of Keplerian elements:
%       [a e i Om om nu], where nu is the true anomaly.
%       a in [L], angles in [rad].
%
%   Vector of Modified Equinoctial Elements:
%       [p f g h k L], where L is the true longitude.
%       p in [L], angles in [rad].
% 
% INPUT:
%	kep[n,6]    Array of Keplerian elements [a e i Om om nu],
%                where nu is the true anomaly. a in [L], angles in [rad].
% 
% OUTPUT:
%	mee[n,6]    Array of Modified Equinoctial Elements [p f g h k L],
%                where L is the true longitude. p in [L], L in [rad].
%
%
% CALLED FUNCTIONS:
%   (none)
% 
% REFERENCES:
%   - Walker,M.J.H., Ireland,B., Owens,J., "A Set of Modified Equinoctial
%       Orbit Elements", Celestial Mechanics Vol. 36 No 4, 1985.
%	- Betts, J. T., "Practical Methods for Optimal Control and Estimation
%       Using Nonlinear Programming", 2010.
%   - De Pascale, P., Vasile, M., "Preliminary Design of Low-Thrust
%       Multiple Gravity-Assist Trajectories", Journal of Spacecraft and
%       Rockets Vol. 43 No 5, 2006.
%
% AUTHOR:
%   Alessandro Peloni, 05/02/2014, MATLAB, kep2mee.m
%
% CHANGELOG:
%   (none)
%
% -------------------------------------------------------------------------


p      = kep(:,1).*(1-kep(:,2).^2);
f      = kep(:,2).*cos(kep(:,4)+kep(:,5));
g      = kep(:,2).*sin(kep(:,4)+kep(:,5));
h      = tan(kep(:,3)/2).*cos(kep(:,4));
k      = tan(kep(:,3)/2).*sin(kep(:,4));
L      = kep(:,4) + kep(:,5) + kep(:,6); % True longitude [rad]

mee = [p f g h k L];

end