function kep = mee2kep(mee)

% mee2kep.m -  Modified Equinoctial Elements to Keplerian elements.
%
% PROTOTYPE:
%   kep = mee2kep(mee)
%
% DESCRIPTION:
%   Transformation from Modified Equinoctial Elements to Keplerian elements.
%   Vector of Modified Equinoctial Elements:
%       [p f g h k L], where L is the true longitude.
%       p in [L], angles in [rad].
%
%   Vector of Keplerian elements:
%       [a e i Om om nu], where nu is the true anomaly.
%       a in [L], angles in [rad].
%   The threshold for inclination is set to 0.1deg, so if incl < 0.1deg -->
%    incl = 0.
% 
% INPUT:
%	mee[6]      Vector of Modified Equinoctial Elements [p f g h k L],
%                where L is the true longitude. p in [L], L in [rad].
% 
% OUTPUT:
%	kep[6,1]    Vector of Keplerian elements [a e i Om om nu],
%                where nu is the true anomaly. a in [L], angles in [rad].
%
%
% CALLED FUNCTIONS:
%   (none)
% 
% REFERENCES:
%   - Walker,M.J.H., Ireland,B., Owens,J., "A Set of Modified Equinoctial
%       Orbit Elements", Celestial Mechanics Vol. 36 No 4, 1985.
%	- Betts, J. T., "Practical Methods for Optimal Control and Estimation
%       Using Nonlinear Programming", 2010.
%   - De Pascale, P., Vasile, M., "Preliminary Design of Low-Thrust
%       Multiple Gravity-Assist Trajectories", Journal of Spacecraft and
%       Rockets Vol. 43 No 5, 2006.
%
% AUTHOR:
%   Alessandro Peloni, 05/12/2013, MATLAB, mee2kep.m
%
% CHANGELOG:
%   27/01/2014, REVISION: Alessandro Peloni
%   03/04/2014, Alessandro Peloni: Change the calculation of Omega and add
%       an alternative way to calculate omega
%
% -------------------------------------------------------------------------


p = mee(1);
f = mee(2);
g = mee(3);
h = mee(4);
k = mee(5);
L = mee(6);


% Eccentricity
kep(2,1) = sqrt(f^2 + g^2); % eccentricity


% Semi-major axis
kep(1,1) = p/(1-kep(2,1)^2); % semi-major axis [L]


% Inclination
kep(3,1) = 2*atan(sqrt(h^2+k^2)); % inclination [rad]
if kep(3,1) < 0 % inclination is defined into [0,pi)
    kep(3,1) = kep(3,1)+pi;
end


% Right Ascension of the Ascending Node
kep(4,1) = atan2(k, h); % RAAN [rad]
kep(4,1) = mod(kep(4,1),2*pi); % RAAN in [0,2pi]


% Argument of perigee
kep(5,1) = atan2(g, f) - kep(4,1); % argument of perigee [rad]
    % Alternative way (note that the definitions of f,g,h,k are slightly
    % different, but are correct in this case because atan2 simply compute
    % sin/cos):
    % f = cos(om+Om) = cos(om)*cos(Om) - sin(om)*sin(Om)
    % g = sin(om+Om) = sin(om)*cos(Om) + cos(om)*sin(Om)
    % h = cos(Om)
    % k = sin(Om)
    % With the definitions above, it can be dimonstrated that:
    % g*h - f*k = sin(om)
    % f*h + g*k = cos(om)
%     kep(5,1) = atan2(g*h-f*k, f*h+g*k); % alternative way
kep(5,1) = mod(kep(5,1),2*pi); % omega in [0,2pi]


% True anomaly
kep(6,1) = L - kep(4,1) - kep(5,1); % true anomaly [rad]


return