function x_eci = sunline2eci(x_sunline,y_ref,lambda)

% sunline2eci.m - Vector reference frame transformation.
%    Sun-line reference frame to ECI reerence frame.
%
% PROTOTYPE:
%   x_eci = sunline2eci(x_sunline,y_ref,lambda)
%
% DESCRIPTION:
%   Transformation from sun-line to ECI reference frame.
%   Sun-line reference frame: {rs,p,rs_p} - as in McInnes, p.115
%       rs-axis: direction of the Sun. The orientation is from the Sun to
%                the spacecraft.
%       p-axis: perpendicular to the sun-line direction, in the plane
%               formed by rs-axis and y_ref-vector.
%       rs_p-axis: completes the reference frame, it is the vector
%                  perpendicular both to rs-axis and y_ref-vector.
%   ECI reference frame: {c1,c2,c3}
%       (c1,c2) plane: equatorial plane.
%       c3-axis: direction of the North Pole from the centre of the Earth.
%
% INPUT:
%	x_sunline[3]    Vector to be transformed, expressed in {rs,p,rs_p}.
%   y_ref[3]        Reference vector to define the sun-line reference
%                    frame, expressed in {c1,c2,c3}.
% 	lambda          Longitude of the Sun [rad].
%
% OUTPUT:
%	x_eci[3,1]      Vector transformed into {c1,c2,c3}.
%
% EXAMPLE:
%   Given a sailcraft in orbit:
%       - we have the sailcraft normal vector in sun-line reference frame;
%       - we want to view the sailcraft attitude respect to the Earth.
%   In this case:
%       x_sunline = Sailcraft normal vector in sun-line reference frame;
%       y_ref = Velocity vector of the spacecraft in {c1,c2,c3};
%       x_eci = Sailcraft normal vector, transformed in {c1,c2,c3}.
%
% CALLED FUNCTIONS:
%   crossFast
%
% AUTHOR:
%   Alessandro Peloni, 11/05/2013, MATLAB, sunline2eci.m
%
% CHANGELOG:
%   25/06/2013, Alessandro Peloni: change the sign of the sun-line unit
%       vector.
%
% -------------------------------------------------------------------------

x_sunline = x_sunline(:);

epsilon = 23.5*pi/180; % Obliquity of the equator on the ecliptic plane [rad]

% Unit vector from the Sun to the spacecraft
rs = [cos(lambda) sin(lambda)*cos(epsilon) sin(lambda)*sin(epsilon)]';

rs_p = crossFast(rs,y_ref);
rs_p = rs_p/norm(rs_p);

p = crossFast(rs_p,rs);
p = p/norm(p);

R = [rs p rs_p];

x_eci = R*x_sunline;

return