function sd = dyn_EV_u(t,s,u,K)

% dyn_EV_u.m - Equation of the Variation of Equinoctial Variables 
%   with acceleration control vector u in Orbital reference frame 
%   {r, t, h}.
%
% PROTOTYPE:
%   sd = dyn_EV_u(t,s,u,K)
% 
% DESCRIPTION:
%   Computes the derivative of the Equinoctial Variables for
%   the two-body problem dynamics with control acceleration, in Orbital
%   reference frame {r, t, h}.
%
% INPUT:
%	t[1]    Time [T].
%	s[6]    Equinoctial Variables [a P1 P2 Q1 Q2 l]. a in [m], l in [rad].
%  	u[3]    Control acceleration vector in Orbital coordinates {r,t,h}
%            [m/T^2].
%   K[1]    K parameter [rad]
%
% OUTPUT:
% 	sd[6,1] Derivative of the Equinoctial Variables
%
% CALLED FUNCTIONS:
%   astroConstants
%
% AUTHOR:
%   Alessandro Peloni, 11/05/2013, MATLAB, dyn_EV_u.m
%
% -------------------------------------------------------------------------

u = u(:);

mu = astroConstants(13) * 1e9; % [m^3/s^2]

n = sqrt(mu/s(1)^3);

a_ab = 1/(1+sqrt(1-s(2)^2-s(3)^2));

sL = 1/(1-s(2)*sin(K)-s(3)*cos(K))*((1-a_ab*s(3)^2)*sin(K)+a_ab*s(2)*s(3)*...
    cos(K)-s(2));

cL = 1/(1-s(2)*sin(K)-s(3)*cos(K))*((1-a_ab*s(2)^2)*cos(K)+a_ab*s(2)*s(3)*...
    sin(K)-s(3));

p_r = 1+s(2)*sL+s(3)*cL;

r_h = n*s(1)^2*sqrt(1-s(2)^2-s(3)^2)/(mu*(1+s(2)*sL+s(3)*cL));




sd(1,1) = 2/n/sqrt(1-s(2)^2-s(3)^2)*((s(3)*sL-s(2)*cL)*u(1)+p_r*u(2));

sd(2,1) = r_h*(-p_r*cL*u(1)+(s(2)+(1+p_r)*sL)*u(2)-s(3)*(s(4)*cL-s(5)*sL)*u(3));

sd(3,1) = r_h*(p_r*sL*u(1)+(s(3)+(1+p_r)*cL)*u(2)+s(2)*(s(4)*cL-s(5)*sL)*u(3));

sd(4,1) = r_h/2*(1+s(4)^2+s(5)^2)*sL*u(3);

sd(5,1) = r_h/2*(1+s(4)^2+s(5)^2)*cL*u(3);

sd(6,1) = n-r_h*((a_ab*p_r*(s(2)*sL+s(3)*cL)+2*sqrt(1-s(2)^2-s(3)^2))*u(1)+...
    a_ab*(1+p_r)*(s(2)*cL-s(3)*sL)*u(2)+(s(4)*cL-s(5)*sL)*u(3));

return