function out = astroConstants(in)

% astroConstants.m - Returns astrodynamic-related physical constants.
%
% PROTOTYPE:
%   out = astro_constants(in)
%
% DESCRIPTION:
%   Returns a row vector of constants, in which there is the corresponding
%   constant for each element of the input vector.
%
%   List of identifiers:
%       Generic astronomical constants:
%           1   Universal gravity constant (G) (from DITAN) [km^3/(kg*s^2)]
%           2   Astronomical Unit (AU) (from DITAN) [km]
%       Sun related:
%           3   Sun mean radius (from DITAN) [km]
%           4   Sun planetary constant (mu = mass * G) (from DITAN)
%               [km^3/s^2]
%           31  Energy flux density of the Sun (from Wertz,SMAD)
%               [W/m2 at 1 AU]
%       Other:
%           5   Speed of light in the vacuum (definition in the SI) [km/s]
%           6   Standard free fall (the acceleration due to gravity on the
%               Earth's surface at sea level) (from Wertz,SMAD) [m/s^2]
%           7   Mean distance Earth-Moon (from Wertz,SMAD) [km]
%           8   Obliquity (angle) of the ecliptic at Epoch 2000 (from
%               Wertz,SMAD) [rad]
%           9   Gravitatonal field constant of the Earth (from Wertz,SMAD,
%               taken from JGM-2). This should be used in conjunction to
%               Earth radius = 6378.1363 km 
%       Planetary constants of the planets (mu = mass * G) [km^3/s^2]:
%           11  Me      (from DITAN)
%           12  V       (from DITAN)
%           13  E       (from DITAN)
%           14  Ma      (from DITAN)
%           15  J       (from DITAN)
%           16  S       (from DITAN)
%           17  U       (from DITAN)
%           18  N       (from DITAN)
%           19  P       (from DITAN)
%           20  Moon    (from DITAN)
%       Mean radius of the planets [km]:
%           21  Me      (from DITAN)
%           22  V       (from DITAN)
%           23  E       (from DITAN)
%           24  Ma      (from DITAN)
%           25  J       (from DITAN)
%           26  S       (from DITAN)
%           27  U       (from DITAN)
%           28  N       (from DITAN)
%           29  P       (from DITAN)
%           30  Moon    (from DITAN)
%       GTOC 7 (additional constants and parameters):
%           (Earth's orbital elements in the J2000 heliocentric ecliptic
%           reference frame):
%           32   Semimajor axis a [km]
%           33   Eccentricity e
%           34   Inclination i [rad]
%           35   Longitude of ascending node Omega [rad]
%           36   Argument of periapsis omega [rad]
%           37   Mean anomaly at epoch M, [rad]
%           38   Epoch t [MJD]
%           (Other constants):
%           39  Day [s]
%           40  Year [days]
%           41  "01 January 2021 00:00 UT" [MJD]
%           42  "31 December 2030 24:00 UT" [MJD]
%           (Problem parameters):
%           43  Mother-ship dry mass [kg]
%           44  Mother-ship fuel mass [kg]
%           45  Probe dry mass [kg]
%           46  Probe fuel mass [kg]
%           47  Maximum overall mission duration [years]
%           48  Maximum probe mission duration [years]
%           49  Minimum asteroid stay time [days]
%           50  Isp high thrust [s]
%           51  Isp low thrust [s]
%           52  Maximum hyperbolic excess velocity form Earth [km/s]
%           53  Maximum low thrust level [N]
%           54  Maximum number of impulses [-]
%           55  Constraint on rendezvous position [km]
%           56  Constraint on rendezvous velocity [m/s]
%
%   Notes for upgrading this function:
%       It is possible to add new constants.
%       - DO NOT change the structure of the function, as well as its
%           prototype.
%       - DO NOT change the identifiers of the constants that have already
%           been defined in this function. If you want to add a new
%           constant, use an unused identifier.
%       - DO NOT add constants that can be easily computed starting form
%           other ones (avoid redundancy).
%       Contact the author for modifications.
%
% INPUT:
%   in      Vector of identifiers of required constants.
%
% OUTPUT:
%   out     Vector of constants.
%
% EXAMPLE:
%   astroConstants([2, 4, 26])
%      Returns a row vector in which there is the value of the AU, the Sun
%      planetary constant and the mean radius of Saturn.
%
%   astroConstants(10 + [1:9])
%      Returns a row vector with the planetary constant of each planet.
%
% REFERENCES:
%   - DITAN (Direct Interplanetary Trajectory Analysis), Massimiliano
%       Vasile, 2006.
%	- Wertz J. R., Larson W. J., "Space Mission Analysis and Design", Third
%       Edition, Space Technology Library 2003.
%   - Problem Description for the 7th Global Trajectory Optimisation
%       Competition
%
% CALLED FUNCTIONS:
%   (none)
%
% AUTHOR:
%   Matteo Ceriotti, 2006, MATLAB, astroConstants.m
%
% PREVIOUS VERSION:
%   Matteo Ceriotti, 2006, MATLAB, astro_constants.m, Ver. 1.2
%       - Header and function name in accordance with guidlines.
%
% CHANGELOG:
%   26/10/2006, Camilla Colombo: Updated.
%   22/10/2007, Camilla Colombo: astroConstants(8) added (Obliquity (angle)
%       of the ecliptic at Epoch 2000).
%   02/10/2009, Camilla Colombo: Header and function name in accordance
%       with guidlines.
%   12/11/2010, Camilla Colombo: astroConstants(9) added (J2) Note: the
%       present value of J2 is not consistent with the value of the Earth
%       radius. This value of J2 should be used in conjunction to Earth
%       radius = 6378.1363 km 
%   21/05/2014, Nicola Sullo: Addition and modification of parameters and 
%       constants for GTOC 7  
% -------------------------------------------------------------------------

for i=1:length(in)
    switch in(i)
        case 1
            out(i)=6.67259e-20; % From DITAN
        case 2
            out(i)=1.49597870691e8; % From GTOC7 problem description        %149597870.7; % From DITAN
        case 3
            out(i)=700000; % From DITAN
        case 4
            out(i)=1.32712440018e11; % From GTOC7 problem description       %0.19891000000000E+31*6.67259e-20; % From DITAN
        case 5
            out(i)=299792.458; % Definition in the SI
        case 6
            out(i)=9.80665; % Definition in Wertz, SMAD
        case 7
            out(i)=384401; % Definition in Wertz, SMAD
        case 8
            out(i)=23.43928111*pi/180; % Definition in Wertz, SMAD
        case 9
            out(i)=0.1082626925638815e-2; % Definition in Wertz, SMAD
        case 11
            out(i)=0.33020000000000E+24*6.67259e-20; % From DITAN
        case 12
            out(i)=0.48685000000000E+25*6.67259e-20; % From DITAN
        case 13
            out(i)=0.59736990612667E+25*6.67259e-20; % From DITAN
        case 14
            out(i)=0.64184999247389E+24*6.67259e-20; % From DITAN
        case 15
            out(i)=0.18986000000000E+28*6.67259e-20; % From DITAN
        case 16
            out(i)=0.56846000000000E+27*6.67259e-20; % From DITAN
        case 17
            out(i)=0.86832000000000E+26*6.67259e-20; % From DITAN
        case 18
            out(i)=0.10243000000000E+27*6.67259e-20; % From DITAN
        case 19
            out(i)=0.14120000000000E+23*6.67259e-20; % From DITAN
        case 20
            out(i)=0.73476418263373E+23*6.67259e-20; % From DITAN
        case 21
            out(i)=0.24400000000000E+04; % From DITAN
        case 22
            out(i)=0.60518000000000E+04; % From DITAN
        case 23
            out(i)=0.63781600000000E+04; % From DITAN
        case 24
            out(i)=0.33899200000000E+04; % From DITAN
        case 25
            out(i)=0.69911000000000E+05; % From DITAN
        case 26
            out(i)=0.58232000000000E+05; % From DITAN
        case 27
            out(i)=0.25362000000000E+05; % From DITAN
        case 28
            out(i)=0.24624000000000E+05; % From DITAN
        case 29
            out(i)=0.11510000000000E+04; % From DITAN
        case 30
            out(i)=0.17380000000000E+04; % From DITAN
        case 31
            out(i)=1367; % From Wertz, SMAD
        case 32
            out(i)=0.999988049532578*1.49597870691e8;                       
        case 33
            out(i)=1.671681163160e-2;                                       
        case 34
            out(i)=0.8854353079654e-3*(pi/180);                             
        case 35
            out(i)=175.40647696473*(pi/180);                               
        case 36
            out(i)=287.61577546182*(pi/180);                               
        case 37
            out(i)=257.60683707535*(pi/180);                              
        case 38
            out(i)=54000;                                            
        case 39
            out(i)=86400;                                                   
        case 40
            out(i)=365.25;                                                  
        case 41
            out(i)=59215;                                                   
        case 42
            out(i)=62867;                                                  
        case 43
            out(i)=6000;                                                               
        case 44
            out(i)=12000;                                                              
        case 45
            out(i)=800;                                                               
        case 46
            out(i)=1200;                                                               
        case 47
            out(i)=12;                                                                
        case 48
            out(i)=6;                                                                
        case 49
            out(i)=30;                                                                
        case 50
            out(i)=900;                                                               
        case 51
            out(i)=3000;                                                               
        case 52
            out(i)=6;                                                                
        case 53
            out(i)=0.3;                                                                
        case 54
            out(i)=10;                                                               
        case 55
            out(i)=1000;                                                               
        case 56
            out(i)=1;  
        % Add an identifier and constant here. Prototype:
        % case $identifier$
        %     out(i)=$constant_value$;
        otherwise
            warning('Constant identifier %d is not defined!',in(i));
            out(i)=0;
    end
end

