function y = BulirschStoerPropagator(y0,x,Settings)
%Description:
% The bulirschstoer single-step ODE propagator has come to MATLAB in 
% the form of a MEX adaptation of Juergen Dietel's  Numerical ODE solver.
%
% Functions must be precompiled to run on MATLAB.  Please Compile using the
% following command sequence:
% >> mex BulirschMex.cpp bulirsch.cpp basis_r.cpp t_dgls.cpp vmblock.cpp
%
%Inputs:
%--------------------------------------------------------------------------
%y0                 [1 x M]                 Initial Position and Velocity 
%                                           coordinates in kilometers.
%                                           [X, Y, Z, Vx, Vy, Vz]
%
%x                  [0 ..END]               x vector. The first
%                                           element corresponds to when
%                                           the initial y values occur.  
%                                           This must consist of at least an 
%                                           initial and a final value 
%                                           (i.e [0 86400])
%
%Settings            Struct                 Settings Struct May Contain the
%                                           Following fields:
%                                           -------------------------------
%         PropType   INT                    Options Currently Include:
%                                           13 = Kepler
%         epsabs     double                 Absolute Error Tolerance
%         epsrel     double                 Relative Error Tolerance
%         h          double                 step size in seconds
%         hMax       double                 Maximum step size in seconds
%         fMax       double                 Maximum Number of function
%                                           calls per time step
%         aMax       double                 Maximum Array Allocation Size
%                                           This determines how much memory
%                                           to use
%SupressDiagnostics  logical                This will display additional
%                                           information about the
%                                           integration from the c routine.
%                                           Typically used for debugging
%                                           purposes
%Outputs:
%--------------------------------------------------------------------------
%sol                [N x M]                 y(x) Solution for all x values
%--------------------------------------------------------------------------
% Programmed by Darin Koblick 02/03/2010 (C) All Rights Reserved.
%--------------------------------------------------------------------------

%Set Optimized Numerical ODE Solver Settings
if exist('Settings','var')
    Settings = OverWriteDefaultSettings(LoadDefaultSettings(),Settings);
else
    Settings = LoadDefaultSettings();
end

%Check if singletonOutput should be invoked
if length(x) == 2
   Settings.singletonOutput = true; 
end

[xsteps, y, nSteps] = BulirschMex(Settings.PropType, ...% Function Call to Modified C
    Settings.epsabs, ...                              % Routine Courtesy of J.Dietel 
    Settings.epsrel, ...                              
    y0(:), ...
    Settings.h, ...
    Settings.hMax, ...
    Settings.fMax, ...
    Settings.SupressDiagnostics, ...
    [x(1),x(end)], ...
    Settings.aMax, ...
    Settings.singletonOutput, ...
    Settings.auxdata);            % CL: Alessandro Peloni, 01/04/2014

if ~Settings.singletonOutput
    %Interpolate to find fixed step sizes if they exist
    y = [y0(:),y(:,1:nSteps)];
    [xsteps, idx] = unique([x(1),xsteps(1:nSteps)]');
    y = interp1(xsteps,y(:,idx)',x,'spline');
else
    y = [y0(:),y(:)];
end
end

function Settings = OverWriteDefaultSettings(DefaultSettings,Settings)
DefaultNameList = fieldnames(DefaultSettings);
for i= DefaultNameList(~ismember(DefaultNameList,fieldnames(Settings)))'
    Settings.(cell2mat(i)) = DefaultSettings.(cell2mat(i));
end
end

function Settings = LoadDefaultSettings()
Settings.epsabs = 1e-14;
Settings.epsrel = 1e-8;
Settings.h = 1E9;
Settings.fMax = 1E15;
Settings.PropType = 15;           % CL: Alessandro Peloni, 01/04/2014
Settings.hMax = 1E9;
Settings.aMax = 1E7;
Settings.SupressDiagnostics = true;
Settings.singletonOutput = false;
Settings.auxdata = [0 0 0];   % CL: Alessandro Peloni, 01/04/2014
end

