/***********************************************************************
*                                                                      *
* Test examples for methods to solve first order ordinary systems of   *
* differential equations.                                              *
*                                                                      *
* We supply the right hand sides, their alpha-numeric description and  *
* the exact analytic solution, if known.                               *
*                                                                      *
* When running the main test program, the user can select among the    *
* examples below listed in the vector `beispiel' by using the function *
* dgls_waehlen().                                                      *
*                                                                      *
*                               C++ Release 1.1 By J-P Moreau, Paris.  *
* -------------------------------------------------------------------- *
* REF.: "Numerical Algorithms with C, By Gisela Engeln-Muellges        *
*        and Frank Uhlig, Springer-Verlag, 1996" [BIBLI 11].           *
*                                                                      *
* Release 1.1: added example #12 with one equation only.               *
***********************************************************************/
#include "basis.h"     /*  for  SIN, EXP, COS, NULL, REAL, HALF, TWO, */
                       /*       ONE, FOUR, NINE, THREE, FIVE, SIX     */
#include "t_dgls.h"    /*  for  bsptyp, dgls_waehlen                  */



/* ----------------------- DE system number 0 ----------------------- */

/***********************************************************************
* Compute the value f of the right hand side of a DE system at (x,y).  *
***********************************************************************/
static void dgl0(REAL x, REAL *y, REAL *auxdata, REAL *f)                   /* CL */
{
  f[0] = y[0] * y[1] + COS(x) - HALF * SIN(TWO * x);
  f[1] = y[0] * y[0] + y[1] * y[1] - (ONE + SIN(x));
}

/***********************************************************************
* alpha-numeric description of above system                            *
***********************************************************************/
static char *dgltxt0(void)
{
  return
    "y1' = y1 * y2 + cos(x) - 0.5 * sin(2.0*x)\n"
    "y2' = y1 * y1 + y2 * y2 - (1 + sin(x))\n";
}



/* ----------------------- DE system number 1 ----------------------- */

/***********************************************************************
* Compute the value f of the right hand side of a DE system at (x,y).  *
***********************************************************************/
static void dgl1(REAL x, REAL *y, REAL *auxdata, REAL *f)                   /* CL */
{
  f[0] = y[1];
  f[1] = (ONE + y[1] * y[1]) / y[0];
}

/***********************************************************************
* alpha-numeric description of above system                            *
***********************************************************************/
static char *dgltxt1(void)
{
  return
    "y1' = y2\n"
    "y2' = (1 + y2 * y2) / y1\n";
}



/* ----------------------- DE system number 2 ----------------------- */

/***********************************************************************
* Compute the value f of the right hand side of a DE system at (x,y).  *
***********************************************************************/
static void dgl2(REAL x, REAL *y, REAL *auxdata, REAL *f)                   /* CL */
{
  f[0] = y[1];
  f[1] = -FOUR * y[0] + EXP(x);
}

/***********************************************************************
* alpha-numeric description of above system                            *
***********************************************************************/
static char *dgltxt2(void)
{
  return
    "y1' = y2\n"
    "y2' = -4 * y1 + exp(x)\n";
}



/* ----------------------- DE system number 3 ----------------------- */

/***********************************************************************
* Compute the value f of the right hand side of a DE system at (x,y).  *
***********************************************************************/
static void dgl3(REAL x, REAL *y, REAL *auxdata, REAL *f)                   /* CL */
{
  f[0] = y[1];
  f[1] = y[2];
  f[2] = y[3];
  f[3] = y[4];
  f[4] = ((REAL)45.0 * y[2] * y[3] * y[4] -
          (REAL)40.0 * y[3] * y[3] * y[3]) / (NINE * y[2] * y[2]);
}

/***********************************************************************
* alpha-numeric description of above system                            *
***********************************************************************/
static char *dgltxt3(void)
{
  return
    "y1' = y2\n"
    "y2' = y3\n"
    "y3' = y4\n"
    "y4' = y5\n"
    "y5' = (45 * y3 * y4 * y5 - 40 * y4 * y4 * y4) / (9 * y3 * y3)\n";
}



/* ----------------------- DE system number 4 ----------------------- */

/***********************************************************************
* Compute the value f of the right hand side of a DE system at (x,y).  *
***********************************************************************/
static void dgl4(REAL x, REAL *y, REAL *auxdata, REAL *f)                   /* CL */
{
  f[0] = -y[1];
  f[1] = y[0];
  f[2] = y[4] - y[1] * y[2];
  f[3] = y[0] * y[3] - y[5];
  f[4] = -y[2] - y[1] * y[4];
  f[5] = y[3] + y[0] * y[5];
}

/***********************************************************************
* alpha-numeric description of above system                            *
***********************************************************************/
static char *dgltxt4(void)
{
  return
    "y1' = -y2\n"
    "y2' = y1\n"
    "y3' = y5 - y2 * y3\n"
    "y4' = y1 * y4 - y6\n"
    "y5' = -y3 - y2 * y5\n"
    "y6' = y4 + y1 * y6\n";
}

/***********************************************************************
* Compute the value of the analytic solution y(x) for the above DE     *
* for the initial value problem  y(0) = (1,0,0,1,e,0)                  *
***********************************************************************/
static void loesung4(REAL x, REAL *y)
{
  y[0] = COS(x);
  y[1] = SIN(x);
  y[2] = SIN(x) * EXP(COS(x));
  y[3] = COS(x) * EXP(SIN(x));
  y[4] = COS(x) * EXP(COS(x));
  y[5] = SIN(x) * EXP(SIN(x));
}



/* ----------------------- DE system number 5 ----------------------- */

/***********************************************************************
* Compute the value f of the right hand side of a DE system at (x,y).  *
***********************************************************************/
static void dgl5(REAL x, REAL *y, REAL *auxdata, REAL *f)                   /* CL */
{
  f[0] = (REAL)-500.5 * y[0] + (REAL)499.5 * y[1];
  f[1] =  (REAL)499.5 * y[0] - (REAL)500.5 * y[1];
}

/***********************************************************************
* alpha-numeric description of above system                            *
***********************************************************************/
static char *dgltxt5(void)
{
  return
    "y1' = -500.5 * y1 + 499.5 * y2\n"
    "y2' =  499.5 * y1 - 500.5 * y2\n";
}

/***********************************************************************
* Compute the value of the analytic solution y(x) for the above DE     *
* for the initial value problem  y(0) = (4,2)                          *
***********************************************************************/
static void loesung5(REAL x, REAL *y)
{
  y[0] = THREE * EXP(-x) + EXP((REAL)-1000.0 * x);
  y[1] = THREE * EXP(-x) - EXP((REAL)-1000.0 * x);
}



/* ----------------------- DE system number 6 ----------------------- */

/***********************************************************************
* Compute the value f of the right hand side of a DE system at (x,y).  *
***********************************************************************/
static void dgl6(REAL x, REAL *y, REAL *auxdata, REAL *f)                   /* CL */
{
  f[0] = (REAL)15.0 * y[1];
  f[1] = (REAL)-0.6 * y[0];
}

/***********************************************************************
* alpha-numeric description of above system                            *
***********************************************************************/
static char *dgltxt6(void)
{
  return
    "y1' = 15.0 * y2\n"
    "y2' = -0.6 * y1\n";
}

/***********************************************************************
* Compute the value of the analytic solution y(x) for the above DE     *
* for the initial value problem  y(0) = (0,1)                          *
***********************************************************************/
static void loesung6(REAL x, REAL *y)
{
  y[0] = FIVE * SIN(THREE * x);
  y[1] = COS(THREE * x);
}



/* ----------------------- DE system number 7 ----------------------- */

/***********************************************************************
* Compute the value f of the right hand side of a DE system at (x,y).  *
***********************************************************************/
static void dgl7(REAL x, REAL *y, REAL *auxdata, REAL *f)                   /* CL */
{
  f[0] = y[1];
  f[1] = y[2];
  f[2] = y[3];
  f[3] = FOUR * y[3] - SIX * y[2] + FOUR * y[1] - y[0];
}

/***********************************************************************
* alpha-numeric description of above system                            *
***********************************************************************/
static char *dgltxt7(void)
{
  return
    "y1' = y2\n"
    "y2' = y3\n"
    "y3' = y4\n"
    "y4' = 4.0 * y4 - 6.0 * y3 + 4.0 * y2 - y1\n";
}

/***********************************************************************
* Compute the value of the analytic solution y(x) for the above DE     *
* for the initial value problem  y(0) = (0,0,0,6)                      *
***********************************************************************/
static void loesung7(REAL x, REAL *y)
{
  y[0] = x * x * x * EXP(x);
  y[1] = x * x * (x + THREE) * EXP(x);
  y[2] = x * (x * x + SIX * x + SIX) * EXP(x);
  y[3] = (x * x * x + NINE * x * x + (REAL)18.0 * x + SIX) * EXP(x);
}



/* ----------------------- DE system number 8 ----------------------- */

/***********************************************************************
* Compute the value f of the right hand side of a DE system at (x,y).  *
***********************************************************************/
static void dgl8(REAL x, REAL *y, REAL *auxdata, REAL *f)                   /* CL */
{
  f[0] = (REAL)-52.0 * y[0] + (REAL)50.0 * y[1];
  f[1] =  (REAL)50.0 * y[0] - (REAL)52.0 * y[1];
}

/***********************************************************************
* alpha-numeric description of above system                            *
***********************************************************************/
static char *dgltxt8(void)
{
  return
    "y1' = -52 * y1 + 50 * y2\n"
    "y2' =  50 * y1 - 52 * y2\n";
}

/***********************************************************************
* Compute the value of the analytic solution y(x) for the above DE     *
* for the initial value problem                                        *
*      y(0) = (1265588.55375228,-1265586.55375225)                     *
***********************************************************************/
static void loesung8(REAL x, REAL *y)
{
  y[0] = (REAL)1.000000015       * EXP(        -TWO * x) +
         (REAL)1265587.553752265 * EXP((REAL)-102.0 * x);
  y[1] = (REAL)1.000000015 *       EXP(        -TWO * x) -
         (REAL)1265587.553752265 * EXP((REAL)-102.0 * x);
}



/* ----------------------- DE system number 9 ----------------------- */

/***********************************************************************
* Compute the value f of the right hand side of a DE system at (x,y).  *
***********************************************************************/
static void dgl9(REAL x, REAL *y, REAL *auxdata, REAL *f)                   /* CL */
{
  f[0] = y[1];
#if defined(BC3)
  _fpreset();
#endif
  f[1] = -y[0] * COSH(x);
}


/***********************************************************************
* alpha-numeric description of above system                            *
***********************************************************************/
static char *dgltxt9(void)
{
  return
    "y1' = y2\n"
    "y2' = -y1 * cosh(x)\n";
}


/* ----------------------- DE system number 10  --------------------- */

/***********************************************************************
* Compute the value f of the right hand side of a DE system at (x,y).  *
***********************************************************************/
static void dgl10(REAL x, REAL *y, REAL *auxdata, REAL *f)                  /* CL */
{
  f[0] = y[1];
  f[1] = -y[0] * y[0] * y[0];
}

/***********************************************************************
* alpha-numeric description of above system                            *
***********************************************************************/
static char *dgltxt10(void)
{
  return
    "y1' = y2\n"
    "y2' = -y1^3\n";
}



/* ----------------------- DE system number 11 ---------------------- */
/* --------------------- (stiff IVP for gear()) --------------------- */

/***********************************************************************
* Compute the value f of the right hand side of a DE system at (x,y).  *
***********************************************************************/
static void dgl11(REAL x, REAL *y, REAL *auxdata, REAL *f)                  /* CL */
{
#define dk  ((REAL)-10000.0)
  f[0] = y[1];
  f[1] = dk * y[0] + (dk - ONE) * y[1];
#undef  dk
}

/***********************************************************************
* alpha-numeric description of above system                            *
***********************************************************************/
static char *dgltxt11(void)
{
  return
    "y1' =  y1                                    (dk = -10000)\n"
    "y2' =  dk * y1 + (dk - 1) * y2\n";
}

/***********************************************************************
* Compute the value of the analytic solution y(x) for the above DE     *
* for the initial value problem  (y0) = (1,1)                          *
***********************************************************************/
static void loesung11(REAL x, REAL *y)
{
  y[0] = EXP(-x);
  y[1] = -EXP(-x);
}

/* ----------------------- DE system number 12 ---------------------- */
/*                         (used by tawp.cpp)                         */

/***********************************************************************
* Compute the value f of the right hand side of a DE system at (x,y).  *
***********************************************************************/
static void dgl12(REAL x, REAL *y, REAL *auxdata, REAL *f)                  /* CL */
{
  f[0] = -y[0] + x/((1+x)*(1+x));
}

/***********************************************************************
* alpha-numeric description of above system                            *
***********************************************************************/
static char *dgltxt12(void)
{
  return "y' = -y + x*((1+x)*(1+x))";
}

/***********************************************************************
* Compute the value of the analytic solution y(x) for the above DE     *
* for the initial value problem  (y0) = (1,1)                          *
***********************************************************************/
static void loesung12(REAL x, REAL *y)
{
  y[0] = 1/(1+x);
}

/* ----------------------- DE system number 13 ----------------------- */
/* ----------------------- StateVector for OD ----------------------- */

/***********************************************************************
* Compute the value f of the right hand side of a DE system at (x,y).  *
***********************************************************************/
static void dgl13(REAL x, REAL *y, REAL *auxdata, REAL *f)                  /* CL */
{
  double NegMurMag3 = -(3.986004418E5/pow(sqrt(y[0]*y[0]+y[1]*y[1]+y[2]*y[2]),3));
  f[0] = y[3];
  f[1] = y[4];
  f[2] = y[5];
  f[3] = NegMurMag3*y[0];
  f[4] = NegMurMag3*y[1];
  f[5] = NegMurMag3*y[2];
}

/***********************************************************************
* alpha-numeric description of above system                            *
***********************************************************************/
static char *dgltxt13(void)
{
  return
    "y1' = Vx0\n"
    "y2' = Vy0\n"
    "y3' = Vz0\n"
    "y4' = -(Mu * Rx)/r^3\n"
    "y5' = -(Mu * Ry)/r^3\n"
    "y6' = -(Mu * Rz)/r^3\n";
}


/* ----------------------- DE system number 14 ----------------------- */
/* --- Propagation of the dynamics in Modified Equinoctial Elements -- */
/* --- Alessandro Peloni, 28/03/2014                                   */

/***********************************************************************
* Compute the value f of the right hand side of a DE system at (x,y).  *
***********************************************************************/
static void dgl14(REAL x, REAL *y, REAL *auxdata, REAL *fun)
{
    /* Define threshold for sin and cos:
     * in Matlab, cos(pi/2) ~= 1e-17. With a threshold of 1e-15 on the true
     * longitude L, the error on the orbit is ~ 1AU*1e-15 = 0.15mm     */
    double threshold = 1e-15;
    
    /* Auxiliary data */
    REAL mu    = auxdata[0]; /* Sun Planetary constant [L^3/T^2] */
    REAL AU    = auxdata[1]; /* Astronomical Unit [L] */
    REAL ac    = auxdata[2]; /* Characteristic acceleration [L/T^2] */
    REAL alpha = auxdata[3]; /* Sail cone angle [rad] */
    REAL delta = auxdata[4]; /* Sail clock angle [rad] */
    
    /* Modified Equinoctial Elements */
    REAL p = y[0];
    REAL f = y[1];
    REAL g = y[2];
    REAL h = y[3];
    REAL k = y[4];
    REAL L = y[5];
    
    /* Define auxiliary variables */
    REAL cL = COS(L);
    if(ABS(cL) < threshold) cL = 0;
    REAL sL = SIN(L);
    if(ABS(sL) < threshold) sL = 0;
    REAL q = 1. + f*cL + g*sL;
    REAL r = p/q; /* Magnitude of the position vector [L] */
    REAL s2 = 1. + h*h + k*k;
    
    REAL pmu = SQRT(p/mu);
    
    
    /* Disturbing acceleration */
    REAL calpha = COS(alpha);
    REAL salpha = SIN(alpha);
    REAL cdelta = COS(delta);
    REAL sdelta = SIN(delta);
    if(ABS(calpha) < threshold) calpha = 0;
    if(ABS(salpha) < threshold) salpha = 0;
    if(ABS(cdelta) < threshold) cdelta = 0;
    if(ABS(sdelta) < threshold) sdelta = 0;
    REAL Delta1 = ac*(AU/r)*(AU/r)*calpha*calpha*calpha;
    REAL Delta2 = ac*(AU/r)*(AU/r)*calpha*calpha*salpha*cdelta;
    REAL Delta3 = ac*(AU/r)*(AU/r)*calpha*calpha*salpha*sdelta;

    
    /* Differential equations of motion */
    fun[0] = (2*p/q)*pmu*Delta2;

    fun[1] = pmu*sL*Delta1 + pmu*(1/q)*((q+1)*cL+f)*Delta2 - pmu*(g/q)*(h*sL-k*cL)*Delta3;

    fun[2] = -pmu*cL*Delta1 + pmu*(1/q)*((q+1)*sL+g)*Delta2 + pmu*(f/q)*(h*sL-k*cL)*Delta3;

    fun[3] = pmu*(s2*cL/(2*q))*Delta3;

    fun[4] = pmu*(s2*sL/(2*q))*Delta3;

    fun[5] = pmu*(1/q)*(h*sL-k*cL)*Delta3 + SQRT(mu*p)*((q/p)*(q/p));
    
}

/***********************************************************************
* alpha-numeric description of above system                            *
***********************************************************************/
static char *dgltxt14(void)
{
  return
    "Differential equations in Modified Equinoctial Elements as in Betts,2010 (p.265)\n";
}


/* ----------------------- DE system number 15 ----------------------- */
/* -------- 2D dynamics of a Solar Sail plus the dynamics of --------- */
/* ------------------- the Lagrangian multipliers -------------------- */
/* --- Alessandro Peloni, 07/04/2014                                   */

/***********************************************************************
* Compute the value f of the right hand side of a DE system at (x,y).  *
***********************************************************************/
static void dgl15(REAL x, REAL *y, REAL *auxdata, REAL *f)
{
    
    /* Define threshold for sin and cos:
     * in Matlab, cos(pi/2) ~= 1e-17     */
    #define threshold  ((REAL)1e-10)
    
    /* Auxiliary data */
    REAL mu    = auxdata[0]; /* Sun Planetary constant [L^3/T^2] */
    REAL AU    = auxdata[1]; /* Astronomical Unit [L] */
    REAL ac    = auxdata[2]; /* Characteristic acceleration [L/T^2] */
        
    /* State vector */
    REAL r           = y[0];
    // REAL theta     = y[1];
    REAL u           = y[2];
    REAL v           = y[3];
    REAL lambda_r    = y[4];
    REAL lambda_th   = y[5];
    REAL lambda_u    = y[6];
    REAL lambda_v    = y[7];
    
    /* Control vector */
    REAL lu = lambda_u;
    REAL lv = lambda_v;
    while(ABS(lu)<HALF || ABS(lv)<HALF) {
        lu = TEN*lu;
        lv = TEN*lv;
    }
     while(ABS(lu)>TEN || ABS(lv)>TEN) {
        lu = lu/TEN;
        lv = lv/TEN;
    }
    REAL tat = lv/lu; /* tan(alpha_tilde) */
    REAL alpha = ATAN((-THREE+SQRT(NINE+EIGHT*tat*tat))/(FOUR*tat)); /* [rad] */
//     REAL alpha = atan((-3*lambda_u+SQRT(9*lambda_u*lambda_u+8*lambda_v*lambda_v))/(4*lambda_v)); /* [rad] */
    REAL calpha = COS(alpha);
    REAL salpha = SIN(alpha);
    if(ABS(calpha) < threshold) calpha = ZERO;
    if(ABS(salpha) < threshold) salpha = ZERO;
    REAL ca2 = calpha * calpha;
    REAL ca3 = calpha * ca2;
    
        
    
    /* Differential equations of motion */
    f[0] = u; /* r_dot [L/T] */

    f[1] = v/r; /* theta_dot [rad/T] */

    f[2] = (v*v/r - mu/(r*r)) + (ac*(AU/r)*(AU/r)*ca3); /* u_dot [L/T^2] */

    f[3] = (-u*v/r) + (ac*(AU/r)*(AU/r)*ca2*salpha); /* v_dot [L/T^2] */

    f[4] = lambda_th*v/(r*r) + lambda_u*((v/r)*(v/r)-TWO*mu/(r*r*r)+TWO*ac/r*(AU/r)*(AU/r)*
        ca3) - lambda_v*((u*v)/(r*r)-TWO*ac/r*(AU/r)*(AU/r)*salpha*ca2); /* lambda_r_dot [1/T^2] */

    f[5] = ZERO; /* lambda_th_dot [L/T^2] */

    f[6] = -lambda_r + lambda_v*(v/r); /* lambda_u_dot [1/T] */

    f[7] = -lambda_th/r - TWO*lambda_u*(v/r) + lambda_v*(u/r); /* lambda_v_dot [1/T] */
    
    
    #undef  threshold
}

/***********************************************************************
* alpha-numeric description of above system                            *
***********************************************************************/
static char *dgltxt15(void)
{
  return
    "Differential equations of the 2D motion of a Solar Sail plus the Lagrange multipliers dynamics\n";
}


/* ------------------------------------------------------------------ */

static bsptyp beispiel[] =                  // Vector for registering
  {	{ 2, dgl0,  dgltxt0,  NULL      },      // all above examples
	{ 2, dgl1,  dgltxt1,  NULL      },      // (zero pointer if exact
	{ 2, dgl2,  dgltxt2,  NULL      },      // solution is not known)
	{ 5, dgl3,  dgltxt3,  NULL      },
	{ 6, dgl4,  dgltxt4,  loesung4  },
	{ 2, dgl5,  dgltxt5,  loesung5  },
	{ 2, dgl6,  dgltxt6,  loesung6  },
	{ 4, dgl7,  dgltxt7,  loesung7  },
	{ 2, dgl8,  dgltxt8,  loesung8  },
	{ 2, dgl9,  dgltxt9,  NULL      },
	{ 2, dgl10, dgltxt10, NULL      },
	{ 2, dgl11, dgltxt11, loesung11 },
	{ 1, dgl12, dgltxt12, loesung12 },
    { 6, dgl13, dgltxt13, NULL      },
    { 6, dgl14, dgltxt14, NULL      },      /* CL: Alessandro Peloni, 28/03/2014 */
    { 8, dgl15, dgltxt15, NULL      }       /* CL: Alessandro Peloni, 07/04/2014 */
  };


/***********************************************************************
* This function allows to pick one of the differential equations in    *
* the vector beispiel for test purposes.                               *
* If the parameter `nummer' lies in the valid range for `beispiel',    *
* the address of the corresponding vector entry is returned, otherwise *
* the zero pointer indicates the error.                                *
***********************************************************************/

bsptyp *dgls_waehlen(int nummer)

{
  if (nummer < 0 || nummer >= sizeof(beispiel) / sizeof(*beispiel))
    return NULL;                        /* invalid number for example */

  return &beispiel[nummer];
}

//end of file t_dgls.cpp
