// ----------------------- MODULE tbulirsch.cpp ------------------------

#include "basis.h"         //  for  umleiten, fprintf, stderr, scanf,
                           //       printf, NULL, REAL, LZS, LZP,
                           //       fehler_melden
#include "vmblock.h"       //  for  vmalloc, vmcomplete, vmfree,
                           //       vminit, VEKTOR
#include "t_dgls.h"        //  for  bsptyp, dgls_waehlen


int bul_stoe       /* Extrapolation method for 1st order DE systems ..*/
    (
     REAL      *x,             /* initial x-value/ final x-value .....*/
     REAL      xend,           /* desired end point ..................*/
     int       n,              /* number of DEs ......................*/
     dglsysfnk dgl,            /* right hand side of DE system .......*/
     REAL      y[],            /* initial y-value/ final y-value .....*/
     REAL      epsabs,         /* absolute error bound ...............*/
     REAL      epsrel,         /* relative error bound ...............*/
     REAL      *h,             /* initial/final step size ............*/
     REAL      hmax,           /* maximal step size ..................*/
     int       neu,            /* use an outside given x ? ...........*/
     long      fmax,           /* maximal # of calls of  dgl() .......*/
     long      *aufrufe        /* actual # of calls of dgl() .........*/
    );                         /* error code .........................*/


void main()  {
/***********************************************************************
* Test program for the function prae_korr() from the module ab_mou to  *
* solve a first order ordinary system of differential equations using  *
* the extrapolation method of Bulirsch-Stoer-Gragg.                    *
*                                                                      *
* Scope of program:                                                    *
* =================                                                    *
* The program reads the input data from the file stdin and writes      *
* output onto the file stdout.                                         *
* if the first command line entry exists, it is interpreted to be the  *
* input file and this is assigned to stdin.                            *
* The second command line parameter is treated analogously wrt. stdout.*
* Calls for input are directed to stderr which also collects error     *
* messages.                                                            *
*                                                                      *
* After reading the input, it is put out for control purposes; then    *
* the adaptive initial value solver is executed and the computed       *
* results are put out.                                                 *
*                                                                      *
* To solve a differential equation, please proceed as in example       *
* t_dgls.cpp.                                                          *
*                                                                      *
* Construction of input data files:                                    *
* =================================                                    *
* bspnummer  Number of DE system in t_dgls.cpp                         *
* epsabs     desired absolute error bound                              *
* epsrel     desired relative error bound                              *
* x0         initial x-value                                           *
* y0[0]   \  initial y-value at x0                                     *
* ...      >                                                           *
* y0[n-1] /                                                            *
* h          starting step size                                        *
* fmax       maximal number of calls of right hand side                *
* xend       final desired x-value                                     *
*                                                                      *
* The number n of differential equations follows from the number of    *
* the DE system chosen; it is stored in conjunction with the right     *
* hand side function in t_dgls.cpp.                                    *
* -------------------------------------------------------------------- *
* SAMPLE RUNS:                                                         *
*                                                                      *
* Example: 0                                                           *
* absolute error bound epsabs: 1e-8                                    *
* relative error bound:epsrel: 1e-10                                   *
* initial x-value x0: 0                                                *
* fonction value y0[0] at x0: 0                                        *
* fonction value y0[1] at x0: 0                                        *
* starting step size h: 0.1                                            *
* desired final x-value: 2                                             *
* maximal number of calls for r.h. side: 1000                          *
*                                                                      *
* System of DEs:                                                       *
* --------------                                                       *
* y1' = y1 * y2 + cos(x) - 0.5 * sin(2.0*x)                            *
* y2' = y1 * y1 + y2 * y2 - (1 + sin(x))                               *
*                                                                      *
* Input data:                                                          *
* -----------                                                          *
* Example  =                        0                                  *
* n        =                        2                                  *
* x0       =   0.000000000000000e+000                                  *
* xend     =   2.000000000000000e+000                                  *
* epsabs   =   1.000000000000000e-008                                  *
* epsrel   =   1.000000000000000e-010                                  *
* fmax     =                     1000                                  *
* h        =   1.000000000000000e-001                                  *
* y0[0]    =   0.000000000000000e+000                                  *
* y0[1]    =   0.000000000000000e+000                                  *
*                                                                      *
* Output data:                                                         *
* ------------                                                         *
* error code from bul_stoe():                                0         *
* final step size:                      5.957495583040763e-002         *
* number of evaluations of r.h. side:                      542         *
* final x-value x =                     2.000000000000000e+000         *
*                                                                      *
* approximate solution y1(x) =   9.193162464585068e-002                *
* approximate solution y2(x) =  -1.363855036077281e+000                *
*                                                                      *
* Example: 12                                                          *
* absolute error bound epsabs: 1e-8                                    *
* relative error bound:epsrel: 1e-10                                   *
* initial x-value x0: 0                                                *
* fonction value y0[0] at x0: 1                                        *
* starting step size h: 0.01                                           *
* desired final x-value: 1                                             *
* maximal number of calls for r.h. side: 500                           *
*                                                                      *
* System of DEs:                                                       *
* --------------                                                       *
* y' = -y + x/((1+x)*(1+x))                                            *
*                                                                      *
* Input data:                                                          *
* -----------                                                          *
* Example  =                       12                                  *
* n        =                        1                                  *
* x0       =   0.000000000000000e+000                                  *
* xend     =   1.000000000000000e+000                                  *
* epsabs   =   1.000000000000000e-008                                  *
* epsrel   =   1.000000000000000e-010                                  *
* fmax     =                      500                                  *
* h        =   1.000000000000000e-001                                  *
* y0[0]    =   1.000000000000000e+000                                  *
*                                                                      *
* Output data:                                                         *
* ------------                                                         *
* error code from bul_stoe():                                0         *
* final step size:                      4.509332728978265e-002         *
* number of evaluations of r.h. side:                      218         *
* final x-value x =                     1.000000000000000e+000         *
*                                                                      *
* approximate solution y1(x) =   5.000000001223922e-001                *
*                                                                      *
* 'exact' solution  y1(x)    =   5.000000000000000e-001                *
*                                                                      *
* difference  approximate solution - 'exact' solution:                 *
*    1.22392207480004e-010                                             *
*                                                                      *
* Example:  3                                                          *
* absolute error bound epsabs: 1e-8                                    *
* relative error bound:epsrel: 1e-10                                   *
* initial x-value x0: 0                                                *
* fonction value y0[0] at x0: 1                                        *
* fonction value y0[1] at x0: 1                                        *
* fonction value y0[2] at x0: 1                                        *
* fonction value y0[3] at x0: 1                                        *
* fonction value y0[4] at x0: 1                                        *
* starting step size h: 0.01                                           *
* desired final x-value: 2                                             *
* maximal number of calls for r.h. side: 1200                          *
*                                                                      *
* System of DEs:                                                       *
* --------------                                                       *
* y1' = y2                                                             *
* y2' = y3                                                             *
* y3' = y4                                                             * 
* y4' = y5                                                             *
* y5' = (45 * y3 * y4 * y5 - 40 * y4 * y4 * y4) / (9 * y3 * y3)        *
*                                                                      *
* Input data:                                                          *
* -----------                                                          *
* Example  =                        3                                  *
* n        =                        5                                  *
* x0       =   0.000000000000000e+000                                  *
* xend     =   1.000000000000000e+000                                  *
* epsabs   =   1.000000000000000e-008                                  *
* epsrel   =   1.000000000000000e-010                                  *
* fmax     =                     1200                                  *
* h        =   1.000000000000000e-002                                  *
* y0[0]    =   1.000000000000000e+000                                  *
* y0[1]    =   1.000000000000000e+000                                  *
* y0[2]    =   1.000000000000000e+000                                  *
* y0[3]    =   1.000000000000000e+000                                  *
* y0[4]    =   1.000000000000000e+000                                  *
*                                                                      *
* Output data:                                                         *
* ------------                                                         *
* error code from bul_stoe():                                0         *
* final step size:                      4.093149569772615e-002         *
* number of evaluations of r.h. side:                     1026         *
* final x-value x =                     2.000000000000000e+000         *
*                                                                      *
* approximate solution y1(x) =   6.708203932418990e+000                *
* approximate solution y2(x) =   5.341640786305866e+000                *
* approximate solution y3(x) =   2.414953415437207e+000                *
* approximate solution y4(x) =  -1.448972049304827e+000                *
* approximate solution y4(x) =  -1.448972049032881e+000                *
*                                                                      *
* -------------------------------------------------------------------- *
* REF.: "Numerical Algorithms with C, by Gisela Engeln-Muellges        *
*        and Frank Uhlig, Springer-Verlag, 1996".                      *
*                                                                      *
*                        Visual C++ Release 1.1 By J-P Moreau, Paris.  *
*                                                                      *
* Release 1.1 (12/16/2006): added two more examples.                   *
***********************************************************************/
//Note: to link with files: bulirsch.cpp, basis_r.cpp, t_dgls.cpp, vmblock.cpp
//----------------------------------------------------------------------------	
  REAL   epsabs,
         epsrel,
         x0,
         *y0,
         *yex,           /* [0..n-1] vector: exact solution           */
         h,
         xend;
  long   fmax,         /* maximal number of calls of right hand side  */
                       /* of DE in awp()                              */
         aufrufe;      /* actual number of calls of r h side in awp() */
  int    bspnummer,
         n,
         fehler,         /* error code from awp()                     */
         i;              /* loop counter                              */

  bsptyp *beispiel;      /* pointer to the structure describing the   */
                         /* problem at hand                           */
  void   *vmblock;       /* List of dynamic allocations               */

  /* -------------------- read input data --------------------------- */

  fprintf(stderr, "\nExample: ");
  scanf("%d", &bspnummer);
  if ((beispiel = dgls_waehlen(bspnummer)) == NULL)
  {
    printf("improper example number",
                  0, __FILE__, __LINE__);
    return;
  }

  n = beispiel->n;
  vmblock = vminit();                           /* initialize storage */
  y0  = (REAL *)vmalloc(vmblock, VEKTOR, n, 0);
  yex = (REAL *)vmalloc(vmblock, VEKTOR, n, 0);
  if (! vmcomplete(vmblock))
  {
    printf("lack of memory", 0, __FILE__, __LINE__);
    return;
  }

  fprintf(stderr, "absolute error bound epsabs: ");
  scanf("%"LZS"f", &epsabs);

  fprintf(stderr, "relative error bound epsrel: ");
  scanf("%"LZS"f", &epsrel);

  fprintf(stderr, "initial x-value x0: ");
  scanf("%"LZS"f", &x0);

  for (i = 0; i < n; i++)
  {
    fprintf(stderr, "Function value y0[%d] at x0: ",i);
    scanf("%"LZS"f", y0 + i);
  }

  fprintf(stderr, "starting step size h:  ");
  scanf("%"LZS"f", &h);

  fprintf(stderr, "desired final x-value: ");
  scanf("%"LZS"f", &xend);

  fprintf(stderr, "maximal number of calls for r.h. side: ");
  scanf("%ld", &fmax);


  /* ------------ print out input ----------------------------------- */

  printf("\n"
         "Solution of a first order ordinary differential equations "
         "system\n"
         "=========================================================="
         "======\n"
         "using the extrapolation method of Bulirsch-Stoer-Gragg\n"
         "======================================================\n\n\n"
         "System of DEs:\n"
         "--------------\n"
         "%s\n\n"
         "Input data:\n"
         "-----------\n"
         "Example  = %24d\n"
         "n        = %24d\n"
         "x0       = %24.15"LZP"e\n"
         "xend     = %24.15"LZP"e\n"
         "epsabs   = %24.15"LZP"e\n"
         "epsrel   = %24.15"LZP"e\n"
         "fmax     = %24ld\n"
         "h        = %24.15"LZP"e\n",
         (*beispiel->dgl_text)(), bspnummer, n, x0, xend, epsabs,
         epsrel, fmax, h);

  for (i = 0; i < n; i++)
    printf("y0[%d]    = %24.15"LZP"e\n", i, y0[i]);


  /* ------------ solve DE system ----------------------------------- */
  fehler =  bul_stoe(&x0,xend,n,beispiel->rechte_seite,y0,epsabs,epsrel,&h,ONE,TRUE,fmax,&aufrufe);
 
  if (fehler != 0)
  {
    printf("Error in bul_stoe #%d\n", 10 + fehler);
    return;
  }


  /* -------------------- print results ----------------------------- */

  printf("\n\n"
         "Output data:\n"
         "------------\n"
         "error code from bul_stoe():         %24d\n"
         "final step size:                    %24.15"LZP"e\n"
         "number of evaluations of r.h. side: %24ld\n"
         "final x-value x =                   %24.15"LZP"e\n\n",
         fehler, h, aufrufe, x0);

  for (i = 0; i < n; i++)
    printf("approximate solution y%d(x) = %24.15"LZP"e\n",
           i + 1, y0[i]);

  if (beispiel->exakte_loesung != NULL)  /* exact solution available? */
  {
    (*beispiel->exakte_loesung)(x0, yex);
    printf("\n");
    for (i = 0; i < n; i++)
      printf("'exact' solution  y%d(x)    = %24.15"LZP"e\n",
             i + 1, yex[i]);
    printf("\ndifference  approximate solution - 'exact' solution:\n");
    for (i = 0; i < n; i++)
      printf("%24.15"LZP"g\n", y0[i] - yex[i]);
  }
  return;  
}

// ----------------------- END  tbulirsch.cpp --------------------------
