function testBS()
%Purpose:
% Test the BulirschStoerPropagator with a planteray orbit
% over a huge time span to check if the final orbit is actually an orbit,
% or becomes a spiral.
%

if exist('BulirschMex','file') ~= 3
    disp('Please Wait ... Compiling BulirschMex Code');
    mex BulirschMex.cpp bulirsch.cpp basis_r.cpp t_dgls.cpp vmblock.cpp;
    disp('Done ... Resuming Code Execution');
end

time = tic;

%Declared Constants
% mu = astroConstants(13); % Earth [km^3/s^2]
mu = astroConstants(4); % Sun [km^3/s^2]
AU = astroConstants(2); % Astronomical Unit [km]
ac = 3e-7; % Characteristic acceleration [km/s^2]
alpha = pi/2;
delta = pi/2;

%Orbital Properties of the Test Case
% r0 = 42164;                                 %Radius in km
r0 = AU;                                    %Radius in km
v0 = sqrt(mu/r0);                           %Velocity in km/s
Period = 2*pi*sqrt(r0^3/mu);                %Period in seconds
eta0(1,1) = r0;                             %Initial x position in km
eta0(1,2) = 0;                              %Initial y position in km
eta0(1,3) = 0;                              %Initial z position in km
eta0(1,4) = 0;                              %Initial x velocity in km/s
eta0(1,5) = v0;                             %Initial y velocity in km/s
eta0(1,6) = 0;                              %Initial z velocity in km/s


% Convert initial position from Cartesian Coordinates into Modified
% Equinoctial Elements
eta0_kep = car2kep(eta0,mu);
eta0_mee = kep2mee(eta0_kep);

%Set Propagation to n orbit revolutions
n = 1000;
t = linspace(0,n*Period,1e3*n);

%Declare acceptable errors for BulirschStoer
Settings.epsabs = 1e-14;
Settings.epsrel = 1e-12;


%Test BulirschStoer Propagation using kepler two body
Settings.PropType = 14;                     %Type 14 = 2BP with Solar Sail in MEE
Settings.h = 1;                             %Initial Starting Step Size Guess
Settings.fMax = 1E9;                        %Maximum number of function iterations
Settings.hMax = 1E9;                        %Maximum step size in seconds
Settings.aMax = 1E7;                        %Maximum number of elements to allocate
                                            %As propogation time gets
                                            %longer this setting must get
                                            %larger
Settings.singletonOutput = false;           %If set to true, aMax can be set
                                            %to one, but intermediate steps
                                            %will not be preserved ... just
                                            %the final integration solution                                     
Settings.SuppressOutput = true;             %Do not print diagnostics to the sceen
Settings.auxdata = [mu AU ac alpha delta];

disp('Testing BS solution...'); tic;

[tsteps, Out_BS, nSteps] = BulirschMex(Settings.PropType, ...% Function Call to Modified C
    Settings.epsabs, ...                              % Routine Courtesy of J.Dietel 
    Settings.epsrel, ...                              
    eta0_mee(:), ...
    Settings.h, ...
    Settings.hMax, ...
    Settings.fMax, ...
    Settings.SuppressOutput, ...
    [t(1),t(end)], ...
    Settings.aMax, ...
    Settings.singletonOutput, ...
    Settings.auxdata);            % CL: Alessandro Peloni, 01/04/2014

if ~Settings.singletonOutput
    %Interpolate to find fixed step sizes if they exist
    Out_BS = [eta0_mee(:),Out_BS(:,1:nSteps)];
    [tsteps, idx] = unique([t(1),tsteps(1:nSteps)]');
    Out_BS = interp1(tsteps,Out_BS(:,idx)',t,'spline');
else
    Out_BS = [eta0_mee(:),Out_BS(:)];
end


disp(['Test Took: ',num2str(toc),' seconds to complete']);
% Clear the BS mex workspace for further simulations
clear BulirschMex.mexw64

% Convert results from MEE to Cartesian Coordinates
kep_BS = zeros(length(t),6);
PosVel_BS = zeros(length(t),6);
disp(' ')
disp('Modified Equinoctial Elements to Cartesian coordinates conversion ...')
for ind = 1:length(t)
    kep_BS(ind,:) = mee2kep(Out_BS(ind,:));
    PosVel_BS(ind,:) = kep2car(kep_BS(ind,:),mu);
end

r = sqrt(sum(PosVel_BS(:,1:3).*PosVel_BS(:,1:3),2)); %Radius of the orbit in km
r = r/r0;
v = sqrt(sum(PosVel_BS(:,4:6).*PosVel_BS(:,4:6),2)); %Velocity of the orbit in km/s
v = v/v0;


% FIGURE:  X, Y, Z over time
labelsXYZ{1,1} = 'x position (km)'; labelsXYZ{2,1} = 'y position (km)'; labelsXYZ{3,1} = 'z position (km)';
labelsXYZ{1,2} = 'time (sec)';      labelsXYZ{2,2} = 'time (sec)';      labelsXYZ{3,2} = 'time (sec)';
labelsXYZ{1,3} = 'x position vs time'; labelsXYZ{2,3} = 'y position vs time'; labelsXYZ{3,3} = 'z position vs time';

figure('color',[1 1 1]);
for i=1:3
eval(['subplot(3,1,',num2str(i),'); plot(t,PosVel_BS(:,', ...
    num2str(i),'),''xr'');']);
xlabel(labelsXYZ{i,2});
ylabel(labelsXYZ{i,1});
title(labelsXYZ{i,3});
grid on
end


% FIGURE:  radius, velocity over time
labelsRV{1,1} = 'normalized radius'; labelsRV{2,1} = 'normalized velocity';
labelsRV{1,2} = 'time (sec)';      labelsRV{2,2} = 'time (sec)';
labelsRV{1,3} = 'radius vs time'; labelsRV{2,3} = 'velocity vs time';

figure('color',[1 1 1]);
eval('subplot(2,1,1); plot(t,r,''xr'');');
xlabel(labelsRV{1,2});
ylabel(labelsRV{1,1});
title(labelsRV{1,3});
grid on

eval('subplot(2,1,2); plot(t,v,''xr'');');
xlabel(labelsRV{2,2});
ylabel(labelsRV{2,1});
title(labelsRV{2,3});
grid on


% FIUGURE:  orbit
figure
plot3(PosVel_BS(:,1), PosVel_BS(:,2), PosVel_BS(:,3));
xlabel('X  [km]')
ylabel('Y  [km]')
zlabel('Z  [km]')
title(['3D orbit propagation: ', num2str(n),' orbit revolutions'])
grid on
axis equal

time = toc(time);
disp(['Total Run Took: ',num2str(time),' seconds to complete']);
