
function testBS15


if exist('BulirschMex','file') ~= 3
    disp('Please Wait ... Compiling BulirschMex Code');
    mex BulirschMex.cpp bulirsch.cpp basis_r.cpp t_dgls.cpp vmblock.cpp;
    disp('Done ... Resuming Code Execution');
end


AU                  = astroConstants(2); % Astronomical Unit [km]
mu                  = astroConstants(4); % Sun planetary constant [km^3/s^2]
ac                  = 0.3e-6; % Characteristic acceleration [km/s^2]
n                   = 1000; % Number of points for the trajectory discretization
id1                 = 3; % Earth



% Scaling factors
scales.length       = AU;
scales.speed        = sqrt(mu/scales.length);
scales.time         = scales.length/scales.speed;
scales.acceleration = scales.speed/scales.time;
scales.gravparam    = scales.acceleration*scales.length^2;


ToF = 600 *86400/scales.time; % [T]
AU = AU/scales.length; % [L]
mu = mu/scales.gravparam; % [L^3/T^2]
ac = ac/scales.acceleration; % [L/T^2]


%--------------------------------------------------------------------------
% Initial state
%--------------------------------------------------------------------------
t0_mjd2000 = date2mjd2000([2019,11,10,0,0,0]); % 10/11/2019 00hr [MJD2000]
t0 = t0_mjd2000*86400/scales.time; % [T]
kep0 = ephSS_kep(id1, t0_mjd2000); % length in [km]
kep0(1) = kep0(1)/scales.length; % length in [L]
kep0(3) = 0; % Hp. planar orbit
car0  = kep2car(kep0,mu); % r in [L], v in [L/T]
v0_car = car0(4:6); % Initial velocity in Cartesian coordinates [L/T]
v0_rth = car2rth(v0_car,car0); % Initial velocity in {r,th,h} frame [L/T]
r0 = sqrt(sum(car0(1:3).*car0(1:3))); % Initial radius [L]
theta0 = kep0(4) + kep0(5) + kep0(6); % Initial angle [rad]
u0 = v0_rth(1); % Initial radial velocity [L/T]
v0 = v0_rth(2); % Initial tangential velocity [L/T]

rng('shuffle')
lambda_r0 = rand(100,1);
rng('shuffle')
lambda_th0 = rand(100,1);
rng('shuffle')
lambda_u0 = rand(100,1);
rng('shuffle')
lambda_v0 = rand(100,1);


%--------------------------------------------------------------------------
% Propagation of the trajectory
%--------------------------------------------------------------------------
for ind = 1:100
    disp(['Iteration No. ', num2str(ind)])
DataIn = [r0, theta0, u0, v0, lambda_r0(ind), lambda_th0(ind), lambda_u0(ind), lambda_v0(ind)];
tspan = linspace(t0,t0+ToF,n);


% Setup for BulirschStoer integrator
Settings.epsabs = 1e-14;
Settings.epsrel = 1e-12;
Settings.PropType = 15;                     % Type 15 = 2BP with Solar Sail in 
                                            % 2D polar coordinates
Settings.h = 1;                             % Initial Starting Step Size Guess
Settings.fMax = 1E9;                        % Maximum number of function iterations
Settings.hMax = 1E9;                        % Maximum step size in seconds
Settings.aMax = 1E7;                          % Maximum number of elements to allocate
                                            % As propogation time gets
                                            % longer this setting must get
                                            % larger
Settings.singletonOutput = false;           % If set to true, aMax can be set
                                            % to one, but intermediate steps
                                            % will not be preserved ... just
                                            % the final integration solution                                     
Settings.SuppressOutput = true;             % Do not print diagnostics to the sceen
Settings.auxdata = [mu AU ac];

[tsteps, y, nSteps] = BulirschMex(Settings.PropType, ...
    Settings.epsabs, ...
    Settings.epsrel, ...                              
    DataIn(:), ...
    Settings.h, ...
    Settings.hMax, ...
    Settings.fMax, ...
    Settings.SuppressOutput, ...
    [tspan(1),tspan(end)], ...
    Settings.aMax, ...
    Settings.singletonOutput, ...
    Settings.auxdata);

if ~Settings.singletonOutput
    % Interpolate to find fixed step sizes if they exist
    y = [DataIn(:),y(:,1:nSteps)];
    [tsteps, idx] = unique([tspan(1),tsteps(1:nSteps)]');
    y = interp1(tsteps,y(:,idx)',tspan,'spline');
else
    y = [DataIn(:),y(:)]';
end

% t = tspan;

x1(:,ind)  = y(:,1); % r [L]
x2(:,ind)  = y(:,2); % theta [rad]
x3(:,ind)  = y(:,3); % u [L/T]
x4(:,ind)  = y(:,4); % v [L/T]
lr(:,ind)  = y(:,5); % lambda_r [1/T]
lth(:,ind) = y(:,6); % lambda_theta [L/T]
lu(:,ind)  = y(:,7); % lambda_u [/]
lv(:,ind)  = y(:,8); % lambda_v [/]
% tf   = t(end); % Transfer time [T]

alpha(:,ind) = atan((-3*lu(:,ind)+sqrt(9*lu(:,ind).^2+8*lv(:,ind).^2))./(4*lv(:,ind))); % [rad]

% Clear the BS mex workspace for further simulations
clear BulirschMex.mexw64

end

save testBS15.mat x1 x2 x3 x4 lr lth lu lv alpha