function testBS()
%Purpose:
% Test the BulirschStoerPropagator with a planteray orbit
% over a huge time span to check if the final orbit is actually an orbit,
% or becomes a spiral.
%

%Declared Constants
mu = astroConstants(13); % Earth [km^3/s^2]

%Orbital Properties of the Test Case
r0 = 42164;                                 %Radius in km
v0 = sqrt(mu/r0);                           %Velocity in km/s
Period = 2*pi*sqrt(r0^3/mu);                %Period in seconds
eta0(1,1) = r0;                             %Initial x position in km
eta0(1,2) = 0;                              %Initial y position in km
eta0(1,3) = 0;                              %Initial z position in km
eta0(1,4) = 0;                              %Initial x velocity in km/s
eta0(1,5) = v0;                             %Initial y velocity in km/s
eta0(1,6) = 0;                              %Initial z velocity in km/s

%Set Propagation to exactly 1000 orbit revolutions
t = linspace(0,1e4*Period,1e4);

%Declare acceptable errors for ODE113 and BulirschStoer
Settings.epsabs = 1e-14;
Settings.epsrel = 1e-12;


%Test BulirschStoer Propagation using kepler two body
Settings.PropType = 13;                     %Type 13 = Kepler Two Body
Settings.h = 1;                             %Initial Starting Step Size Guess
Settings.fMax = 1E9;                        %Maximum number of function iterations
Settings.hMax = 1E9;                        %Maximum step size in seconds
Settings.aMax = 1E7;                        %Maximum number of elements to allocate
                                            %As propogation time gets
                                            %longer this setting must get
                                            %larger
Settings.singletonOutput = false;           %If set to true, aMax can be set
                                            %to one, but intermediate steps
                                            %will not be preserved ... just
                                            %the final integration solution                                     
Settings.SupressDiagnstics = true;          %Do not print diagnostics to the sceen

disp('Testing BS solution...'); tic;
[PosVel_BS] = ...
    BulirschStoerPropagator(eta0(1,1:6),t,Settings);
disp(['Test Took: ',num2str(toc),' seconds to complete']);
r = sqrt(sum(PosVel_BS(:,1:3).*PosVel_BS(:,1:3),2)); %Radius of the orbit in km
r = r/r0;
v = sqrt(sum(PosVel_BS(:,4:6).*PosVel_BS(:,4:6),2)); %Velocity of the orbit in km/s
v = v/v0;


% FIGURE:  X, Y, Z over time
labelsXYZ{1,1} = 'x position (km)'; labelsXYZ{2,1} = 'y position (km)'; labelsXYZ{3,1} = 'z position (km)';
labelsXYZ{1,2} = 'time (sec)';      labelsXYZ{2,2} = 'time (sec)';      labelsXYZ{3,2} = 'time (sec)';
labelsXYZ{1,3} = 'x position vs time'; labelsXYZ{2,3} = 'y position vs time'; labelsXYZ{3,3} = 'z position vs time';

figure('color',[1 1 1]);
for i=1:3
eval(['subplot(3,1,',num2str(i),'); plot(t,PosVel_BS(:,', ...
    num2str(i),'),''xr'');']);
xlabel(labelsXYZ{i,2});
ylabel(labelsXYZ{i,1});
title(labelsXYZ{i,3});
grid on
end


% FIGURE:  radius, velocity over time
labelsRV{1,1} = 'normalized radius'; labelsRV{2,1} = 'normalized velocity';
labelsRV{1,2} = 'time (sec)';      labelsRV{2,2} = 'time (sec)';
labelsRV{1,3} = 'radius vs time'; labelsRV{2,3} = 'velocity vs time';

figure('color',[1 1 1]);
eval('subplot(2,1,1); plot(t,r,''xr'');');
xlabel(labelsRV{1,2});
ylabel(labelsRV{1,1});
title(labelsRV{1,3});
grid on

eval('subplot(2,1,2); plot(t,v,''xr'');');
xlabel(labelsRV{2,2});
ylabel(labelsRV{2,1});
title(labelsRV{2,3});
grid on


% FIUGURE:  orbit
figure
plot3(PosVel_BS(:,1), PosVel_BS(:,2), PosVel_BS(:,3));
xlabel('X  [km]')
ylabel('Y  [km]')
zlabel('Z  [km]')
title('3D orbit propagation: 1000 orbit revolutions')
grid on
axis equal
