function testDriver()
%Purpose:
% Compare the runtime and output results against a real world satellite in
% Low Earth Orbit in order to demonstrate the benefits of running the
% BulirschStoerPropagator vs ODE113
%
%--------------------------------------------------------------------------
% Programmed by Darin Koblick 02/20/2011, All Rights Reserved
%--------------------------------------------------------------------------
%Declared Constants
mu_Earth = num2str(-3.986004418E5);
ft2km = 0.0003048;

%Orbital Properties of LEO Test Case
Period = 5404.13510403513;                  %Period in seconds
eta0(1,1) = -19472500.3.*ft2km;             %Initial x position in km
eta0(1,2) = 6587457.02.*ft2km;              %Initial y position in km
eta0(1,3) = 7367882.5.*ft2km;               %Initial z position in km
eta0(1,4) = -4687.10293.*ft2km;             %Initial x velocity in km/s
eta0(1,5) = -23436.308.*ft2km;              %Initial y velocity in km/s
eta0(1,6) = 8566.3774.*ft2km;               %Initial z velocity in km/s

%Set Propagation to exactly 10 orbit revolutions with 60 sec time steps
t = [0:60:Period*2]; t(end) = Period*2;

%Declare acceptable errors for ODE113 and BulirschStoer
Settings.epsabs = 1e-14;
Settings.epsrel = 1e-12;

%Test ODE113 Propagation using kepler two body
etaDot{1} = 'x(4);';
etaDot{2} = 'x(5);';
etaDot{3} = 'x(6);';
etaDot{4} = ['(x(1).*',mu_Earth,')./(sqrt(x(1).^2+x(2).^2+x(3).^2).^3);'];
etaDot{5} = ['(x(2).*',mu_Earth,')./(sqrt(x(1).^2+x(2).^2+x(3).^2).^3);'];
etaDot{6} = ['(x(3).*',mu_Earth,')./(sqrt(x(1).^2+x(2).^2+x(3).^2).^3)'];
f = inline(['[',cell2mat(etaDot),']'],'t','x');

disp('testing ODE113 solution');
options = odeset('RelTol',Settings.epsrel,'AbsTol',Settings.epsabs);
tic;
[~,PosVel_ODE] = ode113(f,t,eta0(1,:),options);
disp(['Test Took: ',num2str(toc),' seconds to complete']);

%Test BulirschStoer Propagation using kepler two body
Settings.PropType = 13;                     %Type 13 = Kepler Two Body
Settings.h = 1;                             %Initial Starting Step Size Guess
Settings.fMax = 1E9;                        %Maximum number of function iterations
Settings.hMax = 1E9;                        %Maximum step size in seconds
Settings.aMax = 1E7;                        %Maximum number of elements to allocate
                                            %As propogation time gets
                                            %longer this setting must get
                                            %larger
Settings.singletonOutput = false;           %If set to true, aMax can be set
                                            %to one, but intermediate steps
                                            %will not be preserved ... just
                                            %the final integration solution                                     
Settings.SupressDiagnstics = true;          %Do not print diagnostics to the sceen

disp('testing BS solution for LEO CASE'); tic;
[PosVel_BS] = ...
    BulirschStoerPropagator(eta0(1,1:6),t,Settings);
disp(['Test Took: ',num2str(toc),' seconds to complete']);

labels{1,1} = 'x position (km)'; labels{2,1} = 'y position (km)'; labels{3,1} = 'z position (km)';
labels{1,2} = 'time (sec)';      labels{2,2} = 'time (sec)';      labels{3,2} = 'time (sec)';
labels{1,3} = 'x position vs time'; labels{2,3} = 'y position vs time'; labels{3,3} = 'z position vs time';

figure('color',[1 1 1]);
for i=1:3
eval(['subplot(3,1,',num2str(i),'); plot(t,PosVel_ODE(:,', ...
    num2str(i),'),''ok'');','hold on; plot(t,PosVel_BS(:,', ...
    num2str(i),'),''xr'');']);
xlabel(labels{i,2});
ylabel(labels{i,1});
title(labels{i,3});
end


%Propogation Speed Test
disp('Propogation Speed Test ...');
% t = [0 Period*50];
t = [0 90*86400];
Settings.singletonOutput = false;

f = inline(['[',cell2mat(etaDot),']'],'t','x');
disp('testing ODE113 solution');
options = odeset('RelTol',Settings.epsrel,'AbsTol',Settings.epsabs);
tic;
[~,PosVel_ODE] = ode113(f,t,eta0(1,:),options);
disp(['Test Took: ',num2str(toc),' seconds to complete']);

disp('testing BS solution for LEO CASE'); tic;
[PosVel_BS] = ...
    BulirschStoerPropagator(eta0(1,1:6),t,Settings);
disp(['Test Took: ',num2str(toc),' seconds to complete']);

