function [SOL] = beamSearchAst(indast, OUTPUT, bw, folder)

% DESCRIPTION :
% this function perform a Beam Search (BS) for solving the multiple
% asteroid rendezvous (MAR) problem. The beam width (BW) is applied w.r.t.
% the last computed Deep Space Manoeuvre (DSM) in the sequence.

LEGS       = OUTPUT.DEP(OUTPUT.DEP(:,3) == OUTPUT.astLISTe(indast,1) , :); % --> select the departing asteroid and extract the info from OUTPUT
tolCostTot = OUTPUT.tols(4);                                               % --> tol. over the total DV (including departing infinity velocity)

% --> apply the BW (over the last DSM)
LEGS = sortrows(LEGS, size(LEGS,2)-1);
if bw < size(LEGS,1)
    LEGS = LEGS(1:bw,:);
end

if ~isempty(LEGS)
    
    % --> create the folder for saving the results
    if nargin > 3 % --> then the folder is specified
        if exist(folder(2:end), 'dir') == 7 % --> then the folder exists on the current path
            name2 = [folder '\SOL_astMOID_' num2str(indast) '_bw_' num2str(bw) '.mat'];
        else % --> then the folder does not exist on the current path
            mkdir(folder(2:end)); % --> create the folder in the current path
            name2 = [folder '\SOL_astMOID_' num2str(indast) '_bw_' num2str(bw) '.mat'];
        end
    else % --> then the folder is not specified --> save in '\results'
        if exist('results', 'dir') == 7 % --> then the folder '\results' exists on the current path
            name2 = ['\results\SOL_astMOID_' num2str(indast) '_bw_' num2str(bw) '.mat'];
        else % --> then the folder '\results' does not exist on the current path
            mkdir('results');
            name2 = ['\results\SOL_astMOID_' num2str(indast) '_bw_' num2str(bw) '.mat'];
        end
    end
    name    = [pwd name2];
    
    t2   = [];
    while ~isempty(LEGS)

        save(name, 'LEGS');

        t1    = tic;
        LEGSp = search(LEGS, OUTPUT, tolCostTot);
        t2    = [t2 toc(t1)]; toc(t1)

        LEGS = LEGSp;

        % --> apply the beam width (over the last partial DV)
        LEGS = sortrows(LEGS, size(LEGS,2)-1);
        if bw < size(LEGS,1)
            LEGS = LEGS(1:bw,:);
        end

    end
    clear LEGSp;
    load(name);
        
    % --> compute the total cost for the missions identified
    costTOT = DVseq(LEGS);
    
    % --> find the best-DV path
    [minCost, row] = min(costTOT);
    LEGrow         = LEGS(row,:);
    [path]         = constructPathFromLEGS(LEGrow, OUTPUT);
    
    % --> save the solution
    SOL.LEGS             = LEGS;
    SOL.costTOT_legs     = costTOT;
    SOL.path_best_DV     = path;
    SOL.cost_best_path   = minCost;
    SOL.tofy_best_path   = (path(end,2) - path(1,2))/365.25;
    SOL.Elapsed_time_sec = sum(t2);
    ids = path(:,1)';
    ids(ids == 3) = [];
    ids(ids == 4) = [];
    SOL.Nasts            = length(ids);
    SOL.EPH              = OUTPUT.EPH;
    
    % --> delete LEGS from the path
    delete(name);
    
    % --> save the SOL
    save(name, 'SOL');
    
else
    disp('No legs available with current options.');
    SOL = [];
end

end