function [OUTPUT] = wrapSim_v2(SOLS, ind, seq1, folder)

% This function computes the similarity measure of one sequence w.r.t.
% all the other sequences in a set.
%
% INPUT :
% - SOLS   : matrix containing the set of solutions
% - ind    : row index of seq1 in SOLS
% - seq1   : sequence to compare
% - folder : specify the folder where you want the results
%
% OUTPUT :
% - structure containing the following information:
%   --> simScoreTot   : vector of similarity scores of the input sequence
%                       w.r.t. all the other solutions in the set
%   --> simScoreAv    : simScoreTot/maxscore (where maxscore is the maximum
%                       similarity score available, i.e. when two sequences 
%                       are identical)
%   --> NNsimScoreAv  : nearest-neighbour similarity score
%   --> seqScores     : sum(simScoreTot)
%   --> seqScoresNorm : seqScores./nsols
%   --> ind           : row index of seq1 in SOLS
%
% -------------------------------------------------------------------------

MAT_sols        = SOLS;
MAT_sols(ind,:) = [];
lengths         = size(SOLS,2);                                         % --> number of asteroids in the sequences
nsols           = size(SOLS,1);                                         % --> total number of solutions
maxscore        = similarityMeasure_seqs([1:1:lengths], [1:1:lengths]); % --> maximum similarity score

% --> compute the similarities
simScoreTot = zeros(size(MAT_sols,1),1);
for inds = 1:size(MAT_sols,1)
    seq2 = MAT_sols(inds,:);
    [simScoreTot(inds,:)] = similarityMeasure_seqs(seq1, seq2);
end
seqScores     = sum(simScoreTot);
seqScoresNorm = seqScores./nsols;

% --> write the output
OUTPUT.simScoreTot   = simScoreTot;
OUTPUT.simScoreAv    = simScoreTot./maxscore;
OUTPUT.NNsimScoreAv  = min(simScoreTot./maxscore); 
OUTPUT.seqScores     = seqScores;
OUTPUT.seqScoresNorm = seqScoresNorm;
OUTPUT.ind           = ind;

% --> save the results --> create the folder for saving the results
if nargin > 3 && ~isempty(folder) % --> then the folder is specified
    if exist(folder(2:end), 'dir') == 7 % --> then the folder exists on the current path
        name2 = [folder '\OUTPUT_' num2str(ind) '.mat'];
    else % --> then the folder does not exist on the current path
        mkdir(folder(2:end)); % --> create the folder in the current path
        name2 = [folder '\OUTPUT_' num2str(ind) '.mat'];
    end
else % --> then the folder is not specified --> save in '\results'
    if exist('results_similarity', 'dir') == 7 % --> then the folder '\results' exists on the current path
        name2 = ['\results_similarity\OUTPUT_' num2str(ind) '.mat'];
    else % --> then the folder '\results' does not exist on the current path
        mkdir('results_similarity');
        name2 = ['\results_similarity\OUTPUT_' num2str(ind) '.mat'];
    end
end
name    = [pwd name2];
save(name, 'OUTPUT');

end
