function [MOID, East, Eear] = MOIDMilisavljevic(oEl, eEl)%#codegen
% function [MOID East Eear] = PhD_MOIDMilisavljevic(oEl, eEl)
%**************************************************************************
% PhD_MOIDMilisavljevic V 1.0 *  FUNCTION :  PhD_MOIDMilisavljevic
%******************************  AUTHOR   :  Daniel Garcia Yarnoz (dgy)
% STRATH/DME/ASCL             *  PROJECT  :  MechanicsAsteroids
% PHD_FUNCTIONS               *  MATLAB   :  V 7.13.0.564 (R2011b)
%**************************************************************************
%
%    XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
%    X                                                                X
%    X                  MOID CALCULATION BY MILISAVLJEVIC             X
%    X                  ----                -------------             X
%    XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
%
%   It calculates the points of minimum orbit intersection distance (there
%   can be up to 4) and their eccentric anomalies.
%   Based on Milisavljevic, S. (2010). "The Proximities of Asteroids and 
%   Critical Points of the Distance Function." 
%   Not finished. Slow??
%
%**************************************************************************
% HISTORY : V 1.0     21/10/2011 -dgy- creation
%           V 1.1     24/10/2011 -dgy- partly vectorize and speed up
%           V 1.2     25/10/2011 -dgy- modify fzero for polinomials
%           V 1.3     28/10/2011 -dgy- sort
%**************************************************************************
% INPUTS  : oEl       (1x6) [AU,-,rad]   Orbital elements of asteroid
%           eEl       (1x6) [AU,-,rad]   Orbital elements of earth (opt.)
%
% OUTPUTS : MOID      (1x6)   [AU]   MOID vector
%           East      (1x6)   [rad]  eccentric anomaly for the asteroid
%           Eear      (1x6)   [rad]  eccentric anomaly for the earth
%
%  *right now 6 points returned, it can be modified to any numbr of points
%  just by changing the value of npoints inside the function. Theoretically
%  only 4 different points can exist
%           
%**************************************************************************

%% 1.0 INITIALIZATION
N         = 1; % num of asteroids considered

tol       = 1e-6;        % tolerance for convergence
tolOld    = pi/180;      % tolerance to stop if falling close to an already 
                         % existing solution (1 degree in this case)

ecc       = oEl(:,2);
smi       = oEl(:,1).*sqrt(1-ecc.^2);

% vector of size sma and smi direction of the pericentre and perpendicular
[P1,Q1]   = PhD_oelem2PQvector(oEl);

% For Earth:

eccE      = eEl(:,2);
smiE      = eEl(:,1).*sqrt(1-eccE.^2);

% vector of size sma and smi direction of the pericentre and perpendicular
[P2,Q2]   = PhD_oelem2PQvector(eEl);

% Accelerate performance, some calculations outside
P1P1      = oEl(:,1).^2; % dot(P,P) = sma^2 by its definition
P2P2      = eEl(:,1).^2;
Q1Q1      = smi.^2;      % dot(Q,Q) = smi^2 by its definition
Q2Q2      = smiE.^2;
difper    = oEl(:,4)+oEl(:,5)-eEl(:,4)-eEl(:,5); %raan+wp-raanE-wpE

% Allocate sizes of arrays
npoints   = 6;   %Number of starting guess points around orbit
MOID      = NaN*zeros(N,npoints);   %Allocate sizes
East      = NaN*zeros(N,npoints);
Eear      = NaN*zeros(N,npoints);
oldE1     = -999*ones(N,1);
% M1        = zeros(N,1);
% M2        = zeros(N,1);


%% 2.0 CALCULATION

% Start with known ecc anomaly E1 = -w_p (then add steps of 2*pi/npoints)
for jj=1:npoints
    E1                     = mod(-oEl(:,5)+(jj-1)*2*pi/npoints,2*pi);
    E2                     = mod(-difper+E1,2*pi); %just a guess
    % Position vector of E1 in first ellipse
    r1                     = PhD_positionEeccPQ(E1,ecc,P1,Q1);
    for iternum=1:1000
        
        r1P2               = dot(r1',P2')';
        r1Q2               = dot(r1',Q2')';
        % Impose perpendicularity (r2-r1)(dr2/dE2)=0
        for kk=1:N
            % Check it is not NaN (falling into an exisiting solution)
            if isnan(E1(kk))
                E2(kk)     = NaN;
            elseif abs(oldE1(kk)-E1(kk))>tol
                rooties    = rootsPerpPol(eccE(kk),r1P2(kk),r1Q2(kk),P2P2(kk),Q2Q2(kk));
                % Position of those eccentric anomalies
                rrooties   = PhD_positionEeccPQ(rooties,eccE(kk),P2(kk,:),Q2(kk,:));
                % Distance r1-roots
                missd      = (ones(length(rooties),1)*r1(kk,:))-rrooties;
                % Keep the closest distance
%                 [M2(kk),indx] = min(dot(missd',missd'));
                [~,indx] = min(dot(missd',missd'));
                E2(kk)     = rooties(indx);
            end
        end
        % Position vector for roots in second ellipse
        r2                 = PhD_positionEeccPQ(E2,eccE,P2,Q2);

        r2P1               = dot(r2',P1')';
        r2Q1               = dot(r2',Q1')';
        % Impose perpendicularity (r1-r2)(dr1/dE1)=0
        for kk=1:N
            % Check it is not converging to a previous value
            if min(abs(East(kk,:)-E1(kk)))<tolOld || isnan(E1(kk))
                E1(kk)     = NaN;
            % If not already converged, calculate next iteration
            elseif abs(oldE1(kk)-E1(kk))>tol
                oldE1(kk)  = E1(kk);
                rooties    = rootsPerpPol(ecc(kk),r2P1(kk),r2Q1(kk),P1P1(kk),Q1Q1(kk));
                % position of those eccentric anomalies
                rrooties   = PhD_positionEeccPQ(rooties,ecc(kk),P1(kk,:),Q1(kk,:));
                % distance r1-roots
                missd      = (ones(length(rooties),1)*r2(kk,:))-rrooties;
                % keep the closest distance
%                 [M1(kk),indx] = min(dot(missd',missd'));
                [~,indx] = min(dot(missd',missd'));
                E1(kk)     = rooties(indx);
%                 % If distances growing: going towards maximum, discard
%                 if M1(kk) > M2(kk) %apparently this check is bever active?
%                     E1(kk) = NaN;
%                 end
            end
        end
        % Position vector of E1 in first ellipse
        r1                 = PhD_positionEeccPQ(E1,ecc,P1,Q1);
        
        % Check exit condition
        if sum(abs(oldE1-E1)) < N*tol
            break
        end    

            

    end
    % MOID and anomalies
    MOID(:,jj)             = sqrt(dot((r1-r2)',(r1-r2)')');
    East(:,jj)             = mod(E1,2*pi);
    Eear(:,jj)             = mod(E2,2*pi);
end

% Sort outputs in ascending order of Earth eccentric anomaly
[Eear,IX]                  = sort(Eear,2);
for nrow = 1:N
    MOID(nrow,:)           = MOID(nrow,IX(nrow,:));
    East(nrow,:)           = East(nrow,IX(nrow,:));
end

return

%% perpendiCond
function resid = perpendiCond(E,ecc,rP,rQ,PP,QQ)
% function resid = perpendiCond(E, ecc,rP,rQ,PP,QQ)
%**************************************************************************
%
%    XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
%    X                                                                X
%    X               PERPENDICULARITY CONDITION                       X
%    X               -------          ----                            X
%    XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
%
%   Residual of perpendicularity condition between a line and an ellipse at
%   a specific point
%   The line origin is indicated by vector r.
%   The ellipse is defined by two vectors (P along the pericentre with size
%   sma), and Q perpendicular to it with size smi  
%   All inputs except for r refer to one orbit, r is a point in the other
%   orbit
%   PERPENDICULARITY: (r - s)*ds/dE = 0
%                      s     = (cos(E) - ecc) P + sin(E) Q
%                      ds/dE = -sin(E)        P + cos(E) Q
%
%**************************************************************************
% HISTORY : V 1.0     14/10/2011 -dgy- creation
%**************************************************************************
% INPUTS  : E         (Nx1) [rad]  eccentric anomaly
%           ecc       (1x1) [-]    eccentricity
%           rP        (1x1) [AU^2] scalar prod r with pericentre
%           rQ        (1x1) [AU^2] scalar prod r with vector perp to peric
%           PP        (1x1) [AU^2] dot(pericentre, pericentre)=sma
%           QQ        (1x1) [AU^2] dot(Q,Q)
% OUTPUTS : resid     (Nx1) [AU^2] zero if perpenticular
%**************************************************************************

%%
%         r ds/dE  -         s ds/dE
resid = -sin(E)*rP + (cos(E)-ecc).*sin(E)*PP +...
         cos(E)*rQ -  sin(E)     .*cos(E)*QQ ;
return%-------------------------------------------------------------------%

%% rootsPerpPol
function rooties = rootsPerpPol(ecc,rP,rQ,PP,QQ)
% function rooties = rootsPerpPol(ecc,rP,rQ,PP,QQ)
%**************************************************************************
%
%    XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
%    X                                                                X
%    X              ROOTS PERPENDICULARITY POLYNOMIAL                 X
%    X              ----- ----             ---                        X
%    XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
%
%   Roots of the polynomial obtained from the perpendicularity condition 
%   between a line and an ellipse.
%   The line origin is indicated by vector r.
%   The ellipse is defined by two vectors (P along the pericentre with size
%   sma), and Q perpendicular to it with size smi  
%   All inputs except for r refer to the ellipse, r is a point in another
%   orbit for example
%   PERPENDICULARITY: (r - s)*ds/dE = 0
%                      s     = (cos(E) - ecc) P + sin(E) Q
%                      ds/dE = -sin(E)        P + cos(E) Q
%   Polynomial obtained:
%    Assuming sinE = x, cosE = sqrt(1-x^2)
%    4th order poly with coeffs: C^2, 2*B*C, A^2+B^2-C^2, -2*B*C, -B^2
%
%**************************************************************************
% HISTORY : V 1.0     25/10/2011 -dgy- creation
%           V 1.0     27/10/2011 -dgy- remove returning closer root to E
%**************************************************************************
% INPUTS  : ecc       (1x1) [-]    eccentricity
%           rP        (1x1) [AU^2] scalar prod r with pericentre
%           rQ        (1x1) [AU^2] scalar prod r with vector perp to peric
%           PP        (1x1) [AU^2] dot(pericentre, pericentre)=sma
%           QQ        (1x1) [AU^2] dot(Q,Q)
% OUTPUTS : rooties   (Rx1) [rad]  real roots of the polynomial with zero
%                                  residual
%**************************************************************************

%%
% CONDITION:  r ds/dE  -         s ds/dE
% Function A *sinE + B *cosE + C sinE cosE
A       = -rP - ecc*PP;
B       =  rQ;
C       = -QQ + PP;

% Assume sinE = x, cosE = sqrt(1-x^2)
% 4th order polynomial with coeffs: C^2, 2*B*C, A^2+B^2-C^2, -2*B*C, -B^2
polyn   = [C^2 2*B*C A^2+B^2-C^2 -2*B*C -B^2];
xx      = roots(polyn);
%--------------------------------------------------------------------------
% Remove imaginary roots and the ones that cannot be sinE
indexIMG=find(imag(xx)~=0);
if ~isempty(indexIMG)
    xx(indexIMG)=[];
end
indexSinE=find(abs(xx)>1);
if ~isempty(indexSinE)
    xx(indexSinE)=[];
end
%--------------------------------------------------------------------------
rooties = asin(xx);
% pi-rooties can also be a solution  x=sin(E)=sin(pi-E)
% in some cases (C=0) pi+rooties (and also -rooties) can also be a solution
rooties = mod(real([rooties; pi-rooties; -rooties; pi+rooties]), 2*pi);

% Residual: for some roots it may not be zero due to the variable change
resid   = perpendiCond(rooties,ecc,rP,rQ,PP,QQ);

% only valid ones with residual zero
rooties = rooties(abs(resid)<1e-6);

% Root finding close to one removed
% % Eroot: finding the closest to E, take care of 2*pi revolutions
% tmpE    = [rooties; rooties-2*pi; rooties+2*pi];
% [~,indx]= min(abs(tmpE-E));
% Eroot   = tmpE(indx);

                
return%-------------------------------------------------------------------%
%% PhD_oelem2PQvector
function [P,Q] = PhD_oelem2PQvector(oEl)
% function [P,Q] = PhD_oelem2PQvector(oelem)
%**************************************************************************
% PhD_oelem2PQvector V 1.0    *  FUNCTION :  PhD_oelem2PQvector
%******************************  AUTHOR   :  Daniel Garcia Yarnoz (dgy)
% STRATH/DME/ASCL             *  PROJECT  :  MechanicsAsteroids
% PHD_FUNCTIONS               *  MATLAB   :  V 7.13.0.564 (R2011b)
%**************************************************************************
%
%    XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
%    X                                                                X
%    X                P AND Q VECTORS FROM ORBITAL ELEMENTS           X
%    X              --------         -  ---           -     -         X
%    XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
%
%   Simple function to return two vectors P and Q, P in the direction
%   of the pericentre and size sma, Q in the perpendicular direction in the
%   orbital plane and size smi, provided the orbital elements of N orbits
%
%**************************************************************************
% HISTORY : V 1.0     26/10/2011 -dgy- creation
%**************************************************************************
% INPUTS  : oEl    (Nx6)  [AU|km - rad] orbital elements
%
% OUTPUTS : P      (Nx3)  [AU|km]   pericentre vector (size sma)
%           Q      (Nx3)  [AU|km]   perp to peric vector (size smi)
%**************************************************************************

%%
% vector of size sma direction of the pericentre
P   = oEl(:,1)*ones(1,3).*...
    [ cos(oEl(:,5)).*cos(oEl(:,4))-sin(oEl(:,5)).*cos(oEl(:,3)).*sin(oEl(:,4))...
      cos(oEl(:,5)).*sin(oEl(:,4))+sin(oEl(:,5)).*cos(oEl(:,3)).*cos(oEl(:,4))...
                                   sin(oEl(:,5)).*sin(oEl(:,3))              ];
%          ^wp            ^raan         ^wp            ^inc          ^raan    

% vector of size smi direction perpendicular to the pericentre
smi = oEl(:,1).*sqrt(1-oEl(:,2).^2);
Q   = smi*ones(1,3).*...
    [-sin(oEl(:,5)).*cos(oEl(:,4))-cos(oEl(:,5)).*cos(oEl(:,3)).*sin(oEl(:,4))...
     -sin(oEl(:,5)).*sin(oEl(:,4))+cos(oEl(:,5)).*cos(oEl(:,3)).*cos(oEl(:,4))...
                                   cos(oEl(:,5)).*sin(oEl(:,3))            ]; 
%          ^wp            ^raan         ^wp            ^inc          ^raan  
                
return%-------------------------------------------------------------------%
%% PhD_positionEeccPQ
function r = PhD_positionEeccPQ(E,ecc,P,Q)
% function r = PhD_positionEeccPQ(E,ecc,P,Q)
%**************************************************************************
% PhD_positionEeccPQ V 1.0    *  FUNCTION :  PhD_positionEeccPQ
%******************************  AUTHOR   :  Daniel Garcia Yarnoz (dgy)
% STRATH/DME/ASCL             *  PROJECT  :  MechanicsAsteroids
% PHD_FUNCTIONS               *  MATLAB   :  V 7.13.0.564 (R2011b)
%**************************************************************************
%
%    XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
%    X                                                                X
%    X         POSITION VECTOR GIVEN E, ECCENTRICITY, P AND Q         X
%    X              --------         -  ---           -     -         X
%    XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
%
%   Simple function to return the position in an orbit given the eccentric
%   anomaly, the eccentricity, and two vectors P and Q, P in the direction
%   of the pericentre and size sma, Q in the perpendicular direction in the
%   orbital plane and size smi
%   It is a vector function, it can return:
%     - positions for a number N of anomalies at a single orbit
%     - positions for a number N of orbits, one point (one E per orbit) 
%   ecc, P and Q must have the same number of rows.
%   Eccentricity actually not needed (can be obtained from P and Q), but
%   passed along to reduce number of calculations inside a for loop outside
%**************************************************************************
% HISTORY : V 1.0     26/10/2011 -dgy- creation
%**************************************************************************
% INPUTS  : E    (Nx1)       [rad]     eccentric anomaly
%           ecc  (1x1)|(Nx1) [-]       eccentricity
%           P    (1x3)|(Nx3) [AU|km]   pericentre vector (size sma)
%           Q    (1x3)|(Nx3) [AU|km]   perp to peric vector (size smi)
% OUTPUTS : r    (Nx3)       [AU|km]   position vector
%**************************************************************************

%%
% sma     = sqrt(dot(P',P')');
% smi     = sqrt(dot(Q',Q')');
% ecc     = sqrt(1-(smi./sma).^2);
if size(ecc,1) == 1 && size(E,1) ~= 1
    ECC = ones(size(E))*ecc;
    PP   = ones(size(E))*P;
    QQ   = ones(size(E))*Q;
else
    ECC = ecc;
    PP  = P;
    QQ  = Q;
end
toMat   = ones(1,3);
r       = (cos(E)-ECC)*toMat.*PP + sin(E)*toMat.*QQ;
                
return%-------------------------------------------------------------------%
